/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.utils.Transaction;
import org.wso2.registry.jdbc.dao.ResourceVersionDAO;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.List;
import java.util.ArrayList;
import java.sql.Connection;
import java.sql.SQLException;

public class CollectionVersionImpl extends CollectionImpl {

    private static Log log = LogFactory.getLog(CollectionVersionImpl.class);

    private ResourceVersionDAO resourceVersionDAO;

    public CollectionVersionImpl() {
        resourceVersionDAO = new ResourceVersionDAO();
    }

    public String[] getChildren() throws RegistryException {
        return (String[]) getContent();
    }


    public String[] getChildren(int start, int pageLen) throws RegistryException {

        if (content == null) {

            List <String> childPathList;

            if (Transaction.isStarted()) {

                childPathList = resourceVersionDAO.getChildPaths(
                        getId(), snapshotID, versionNumber, Transaction.getConnection());

            } else {

                Connection conn = null;

                try {
                    conn = dataSource.getConnection();
                    childPathList = resourceVersionDAO.
                            getChildPaths(getId(), snapshotID, versionNumber, conn);

                } catch (SQLException e) {

                    String msg = "Failed to get the child paths " + pageLen + " child paths from " +
                            start + " of resource " + getPath() + ". " + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);

                } finally {

                    if (conn != null) {
                        try {
                            conn.close();
                        } catch (SQLException e) {
                            log.error("Failed to close the database connection used to " +
                                    "get child paths of collection " + getPath());
                        }
                    }
                }
            }

            return childPathList.toArray(new String[childPathList.size()]);

        } else {

            if ( content instanceof String[]){

                List<String> childPathList = new ArrayList();
                String childNodes [] = (String[]) content;
                int limit = start + pageLen;
                if (start > childNodes.length) {
                    return new String[0];
                }
                if (limit > childNodes.length) {
                    limit = childNodes.length;
                }
                for (int i = start ; i < limit; i++) {
                    String childNode = childNodes[i];
                    childPathList.add(childNode);
                }
                return childPathList.toArray(new String[childPathList.size()]);
            }
        }

        return new String[0];
    }

    public int getChildCount() throws RegistryException {

        if (childCount == -1) {

            if (Transaction.isStarted()) {

                childCount = resourceVersionDAO.
                        getChildCount(getId(), versionNumber, Transaction.getConnection());

            } else {

                Connection conn = null;

                try {
                    conn = dataSource.getConnection();
                    childCount = resourceVersionDAO.getChildCount(getId(), versionNumber, conn);

                } catch (SQLException e) {

                    String msg = "Failed to get the child count of resource " +
                            getPath() + ". " + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);

                } finally {

                    if (conn != null) {
                        try {
                            conn.close();
                        } catch (SQLException e) {
                            log.error("Failed to close the database connection used to " +
                                    "get child count of collection " + getPath());
                        }
                    }
                }
            }
        }

        return childCount;
    }

    /**
     * Collection's content is a string array, which contains paths of its children. These paths
     * are loaded on demand to increase performance. It is recommended to use getChildren(...)
     * method to get child paths of a collection, which provides pagination. Calling this method
     * will load all child paths.
     *
     * @return String array of child paths.
     * @throws org.wso2.registry.exceptions.RegistryException On any error.
     */
    public Object getContent() throws RegistryException {

        if (content == null) {

            List<String> childPathList = null;

            if (Transaction.isStarted()) {
                childPathList = resourceVersionDAO.getChildPaths(
                        getId(), snapshotID, versionNumber, Transaction.getConnection());

            } else {

                Connection conn = null;

                try {
                    conn = dataSource.getConnection();
                    childPathList = resourceVersionDAO.getChildPaths(
                            getId(), snapshotID, versionNumber, conn);

                } catch (SQLException e) {

                    String msg = "Failed to get child paths of " +
                            getPath() + ". " + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);

                } finally {

                    if (conn != null) {
                        try {
                            conn.close();
                        } catch (SQLException e) {
                            log.error("Failed to close the database connection opened for " +
                                    "accessing the child paths of " + getPath(), e);
                        }
                    }
                }
            }

            content = childPathList.toArray(new String[childPathList.size()]);
        }

        return content;
    }
}
