/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.config;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.store.jdbc.JdbcDirectory;
import org.apache.lucene.store.RAMDirectory;
import org.wso2.registry.Aspect;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.jdbc.Repository;
import org.wso2.registry.jdbc.handlers.HandlerManager;
import org.wso2.registry.jdbc.handlers.CustomEditManager;
import org.wso2.registry.jdbc.utils.RegistryDataSource;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import java.io.InputStream;
import java.util.*;

/**
 * This class provides access to core registry components. Components which should have only one
 * instance are stored in this singleton class. 
 */
public class RegistryContext {

    public interface RegURLSupplier {
        public String getURL();
    }

    private static final Log log = LogFactory.getLog(RegistryContext.class);

    private RegURLSupplier urlSupplier;
    private DataBaseConfiguration defaultDataBaseConfiguration = null;
    private Map<String, DataBaseConfiguration> dbConfigs =
            new HashMap<String, DataBaseConfiguration>();
    private List<HandlerConfiguration> handlerConfigurations = new ArrayList();
    private HandlerManager handlerManager = new HandlerManager();
    private CustomEditManager customEditManager = new CustomEditManager();
    private List mediaTypeHandlers = new ArrayList();
    private List urlHandlers = new ArrayList();
    private List queryProcessors = new ArrayList();
    private Repository repository;
    private Map aspects = new HashMap();
    private DataSource dataSource;
    private EmbeddedRegistry embeddedRegistry;

    private JdbcDirectory jdbcDir;
    private RAMDirectory ramDir;

    private static RegistryContext registryContext = null;

    public static RegistryContext getSingleton() {
        return registryContext;
    }

    public static synchronized void setSingleton(RegistryContext context) {
        registryContext = context;
    }

    public RegistryContext() throws RegistryException {
        this(null);
    }

    public RegistryContext(InputStream configStream) throws RegistryException {
        RegistryConfigurationProcessor.populateRegistryConfig(configStream, this);
    }

    public RegistryContext(InputStream configStream, RegURLSupplier urlSupplier)
            throws RegistryException {
        this.urlSupplier = urlSupplier;
        RegistryConfigurationProcessor.populateRegistryConfig(configStream, this);
    }

    public Repository getRepository() {
        return repository;
    }

    public void setRepository(Repository repository) {
        this.repository = repository;
    }

    public EmbeddedRegistry getEmbeddedRegistry() throws RegistryException {
        if (embeddedRegistry == null) {
            try {
                embeddedRegistry = new EmbeddedRegistry(this);
            } catch (RegistryException e) {
                String msg = "Couldn't initialize EmbeddedRegistry. " + e.getMessage();
                log.error(msg, e);
                throw new RegistryException(msg, e);
            }
        }
        return embeddedRegistry;
    }

    public DataBaseConfiguration getDefaultDataBaseConfiguration() {
        return defaultDataBaseConfiguration;
    }

    public void selectDBConfig(String dbConfigName) {
        DataBaseConfiguration config = dbConfigs.get(dbConfigName);
        if (config == null) {
            // No such configuration...
            log.error("Couldn't find db configuration '" + dbConfigName + "'");
            return;
        }

        final String dataSourceName = config.getDataSourceName();
        if (dataSourceName != null) {
            try {
                Context context = new InitialContext();
                dataSource = (DataSource)context.lookup(dataSourceName);
            } catch (NamingException e) {
                // Problems!
                log.error("Couldn't find dataSource '" + dataSourceName + "'", e);
            }
        } else {
            dataSource = new RegistryDataSource(config.getDbUrl(), config.getDriverName(),
                                                config.getUserName(), config.getPassWord());
        }
    }

    public Iterator<String> getDBConfigNames() {
        return dbConfigs.keySet().iterator();
    }

    public DataBaseConfiguration getDBConfig(String dbName) {
        return dbConfigs.get(dbName);
    }

    public void addDBConfig(String name, DataBaseConfiguration config) {
        // Process "$basedir$" if it's in the URL...
        String url = config.getDbUrl();
        if (url != null) {
            config.setDbUrl(url.replace("$basedir$", getBasePath()));
        }
        dbConfigs.put(name, config);
    }

    public List<HandlerConfiguration> getHandlerConfigurations() {
        return handlerConfigurations;
    }

    public void setHandlerConfigurations(List<HandlerConfiguration> handlerConfigurations) {
        this.handlerConfigurations = handlerConfigurations;
    }

    public void addHandlerConfiguration(HandlerConfiguration handlerConfiguration) {
        handlerConfigurations.add(handlerConfiguration);
    }

    public List getMediaTypeHandlers() {
        return mediaTypeHandlers;
    }

    public void setMediaTypeHandlers(List mediaTypeHandlers) {
        this.mediaTypeHandlers = mediaTypeHandlers;
    }

    public void addMediaTypeHandler(MediaTypeHandlerConfiguration mediaTypeHandlerConfiguration) {
        mediaTypeHandlers.add(mediaTypeHandlerConfiguration);
    }

    public List getUrlHandlers() {
        return urlHandlers;
    }

    public void setUrlHandlers(List urlHandlers) {
        this.urlHandlers = urlHandlers;
    }

    public void addURLHandler(String urlHandler) {
        urlHandlers.add(urlHandler);
    }

    public void addAspect(String name, Aspect aspect) {
        aspects.put(name, aspect);
    }

    public Aspect getAspect(String name) {
        return (Aspect)aspects.get(name);
    }

    public String [] getAspectNames() {
        final Set aspectNames = aspects.keySet();
        return (String [])aspectNames.toArray(new String[aspectNames.size()]);
    }

    public List getQueryProcessors() {
        return queryProcessors;
    }

    public void setQueryProcessors(List queryProcessors) {
        this.queryProcessors = queryProcessors;
    }

    public void addQueryProcessor(QueryProcessorConfiguration queryProcessorConfiguration) {
        queryProcessors.add(queryProcessorConfiguration);
    }

    public String getBasePath() {
        String basePath = null;
        if (urlSupplier != null) {
            basePath = urlSupplier.getURL();
        }
        if (basePath == null) {
            basePath = System.getProperty("basedir", "");
        }
        return basePath;
    }

    public DataSource getDataSource() {
        return dataSource;
    }

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public HandlerManager getHandlerManager() {
        return handlerManager;
    }

    public CustomEditManager getCustomEditManager() {
        return customEditManager;
    }

    public void setCustomEditManager(CustomEditManager customEditManager) {
        this.customEditManager = customEditManager;
    }

    public void setJdbcDir(JdbcDirectory jdbcDir){
        this.jdbcDir = jdbcDir;
    }

    public JdbcDirectory getJdbcDir(){
        return jdbcDir;
    }

    public void setRamDir(RAMDirectory ramDir){
        this.ramDir = ramDir;
    }

    public RAMDirectory getRamDir(){
        return ramDir;
    }
}
