/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.config.RegistryContext;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.jdbc.utils.creators.DatabaseCreator;
import org.wso2.registry.jdbc.utils.creators.DatabaseCreatorFactory;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.users.UserRealm;

import javax.sql.DataSource;

/**
 * This is a core class used by application that use registry in the embedded mode. This class is
 * used to create embedded registry instances for user sessions.
 *
 * UserRegistry is the embedded mode implementation of the Registry API. In this mode, all registry
 * accesses has to be done using a UserRegistry instance. And there has to be separate UserRegistry
 * instance for each user to access the registry. These UserRegistry instances has be obtained
 * from the EmbeddeedRegistry. It is recommended to have only one EmbeddedRegistry instance per
 * application. But there can be exceptions, where it is required to maintain two or more registries
 * pointing to different datasources.
 *
 * Applications should initialize an EmbeddedRegistry instance at the startup using following code.
 *
 * InputStream configStream = new FileInputStream("/projects/registry.xml");
 * RegistryContext registryContext = new RegistryContext(configStream);
 * EmbeddedRegistry embeddedRegistry = new EmbeddedRegistry(registryContext);
 *
 * After initializing an EmbeddedRegistry instance it should be stored in some globally accessible
 * location, so that it can be used by necessary modules to create UserRegistry instances. From
 * this, it is possible to create UserRegistry instances using varous parameter combinations
 * documeneted in getUserRegistry(...) methods.
 *
 * UserRegistry adminRegistry = embeddedRegistry.getUserRegistry("admin", "admin");
 */
public class EmbeddedRegistry {

    private JDBCRegistry jdbcRegistry;
    private UserRealm defaultRealm;
    protected RegistryContext registryContext;

    /**
     * Instantiates the EmbeddedRegistry using the configuration given in RegistryContext.
     * Datasource given in the context will be used for both resource store and the user store.
     *
     * @param context RegistryContext containing the configuration.
     * @throws RegistryException
     */
    public EmbeddedRegistry(RegistryContext context) throws RegistryException {
        this.registryContext = context;
        configure();
    }

    /**
     * Intantiates the EmbeddedRegistry using the configuration given in the context and the given
     * UserRealm. Datasource given in the context will be used for the resource store. User store
     * is accessed from the given UserRealm, which may point to a different source.
     *
     * @param context RegistryContex containing the configuration.
     * @param realm UserRealm containing the user store.
     * @throws RegistryException
     */
    public EmbeddedRegistry(RegistryContext context, UserRealm realm) throws RegistryException {
        this.registryContext = context;
        configure(realm);
    }

    /**
     * This contructor is used by the inherited InMemoryEmbeddedRegistry class as it has to be
     * instantiated using the default contructor.
     */
    protected EmbeddedRegistry() {
    }

    protected void configure() throws RegistryException {
        DataSource dataSource = registryContext.getDataSource();

        DatabaseCreator databaseCreator = DatabaseCreatorFactory.getDatabaseCreator(dataSource);
        databaseCreator.createRegistryDatabase();
        databaseCreator.createUserManagerDatabase();

        if (defaultRealm == null) {
            defaultRealm = new RegistryRealm(dataSource);
        }
        jdbcRegistry = new JDBCRegistry(registryContext, defaultRealm);
    }

    protected void configure(UserRealm defaultRealm) throws RegistryException {
        this.defaultRealm = defaultRealm;
        configure();
    }

    /**
     * Creates a UserRegistry instance for anonymous user. Permissions set for anonymous user will
     * be applied for all operations performed using this instance.
     *
     * @return UserRegistry for the anonymous user.
     * @throws RegistryException
     */
    public UserRegistry getUserRegistry() throws RegistryException {
        return new UserRegistry(
                RegistryConstants.ANONYMOUS_USER,
                RegistryConstants.ANONYMOUS_PASSWORD,
                jdbcRegistry,
                defaultRealm,
                registryContext.getDataSource());
    }

    /**
     * Returns a registry to be used for system operations. Human users should not be allowed
     * log in using this registry.
     *
     * @return User registry for system user.
     * @throws RegistryException
     */
    public UserRegistry getSystemRegistry() throws RegistryException {
        return new UserRegistry(
                RegistryConstants.SYSTEM_USER,
                RegistryConstants.SYSTEM_PASSWORD,
                jdbcRegistry,
                defaultRealm,
                registryContext.getDataSource());
    }

    /**
     * Creates UserRegistry instances for normal users. Applications should use this method to
     * create UserRegistry instances, unless there is a specific need documented in other methods.
     * User name and the password will be authenticated by the EmbeddedRegistry before creating
     * the requested UserRegistry instance.
     *
     * @param userName User name of the user.
     * @param password Password of the user.
     * @return UserRegistry instance for the given user.
     * @throws RegistryException
     */
    public UserRegistry getUserRegistry(String userName, String password) throws RegistryException {
        return new UserRegistry(userName, password, jdbcRegistry, defaultRealm,
                                registryContext.getDataSource());
    }

    /**
     * Creates a UserRegistry instance for the given user. This method will NOT authenticate the
     * user before creating the UserRegistry instance. It assumes that the user is authenticated
     * outside the EmbeddedRegistry.
     *
     * @param userName User name of the user.
     * @return UserRegistry instance for the given user.
     * @throws RegistryException
     */
    public UserRegistry getUserRegistry(String userName) throws RegistryException {
        return new UserRegistry(userName, jdbcRegistry, defaultRealm,
                                registryContext.getDataSource());
    }
}
