/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.store.jdbc.JdbcDirectory;
import org.apache.lucene.store.RAMDirectory;
import org.apache.lucene.index.LuceneUtils;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.config.RegistryContext;
import org.wso2.registry.jdbc.dao.ResourceDAO;
import org.wso2.registry.jdbc.filecache.FileManager;
import org.wso2.registry.jdbc.handlers.HandlerManager;
import org.wso2.registry.jdbc.handlers.builtin.*;
import org.wso2.registry.jdbc.handlers.filters.MediaTypeMatcher;
import org.wso2.registry.jdbc.handlers.filters.URLMatcher;
import org.wso2.registry.jdbc.handlers.filters.Filter;
import org.wso2.registry.jdbc.queries.QueryProcessorManager;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.jdbc.utils.Transaction;
import org.wso2.registry.jdbc.utils.FlushTimer;
import org.wso2.registry.jdbc.indexing.DialectFactory;
import org.wso2.registry.session.CurrentSession;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.accesscontrol.AuthorizingRealm;
import org.wso2.registry.users.accesscontrol.AuthorizingRealmConfig;
import org.wso2.registry.utils.AuthorizationUtils;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.util.Date;
import java.util.Map;
import java.io.IOException;

/**
 * This is a core class of the JDBC based implementation of the Registry. This will be used
 * mostly as the back-end by other Registry implementations. This can use either an in-memory
 * database or a external database configured using data source.
 */
public class JDBCRegistry implements Registry {

    private BasicRegistry basicRegistry;
    private AtomicRegistry atomicRegistry;

    private static final Log log = LogFactory.getLog(JDBCRegistry.class);

    private DataSource dataSource = null;
    private ResourceDAO resourceDAO = null;

    protected Repository repository;
    protected VersionRepository versionRepository;

    private UserRealm defaultRealm = null;
    protected RegistryContext registryContext;

    private UserRealm systemUserRealm;

    private JdbcDirectory jdbcDir;
    private RAMDirectory ramDir;

    /**
     * Default constructor. JDBC registry should be configured using the configure(...) method, if
     * it is instantiated using this constructor.
     */
    public JDBCRegistry() {
    }

    /**
     * Constructs a JDBC registry to be used with secure registries. Default authorizations will be
     * applied on the resources created using this registry.
     *
     * @param registryContext RegistryContext containing our configuration and DataSource.  Note
     *                        that this may or may not be the same data source as the UserManager
     * @param realm      User manager realm to handle authorizations. It is strongly recommended to
     *                   use a realm instance constructed using the RegistryRealmFactory. All
     *                   initial users, roles and authorizations will be set on realms obtained
     *                   using the RegistryRealmFactory.
     * @throws RegistryException : If something went wrong
     */
    public JDBCRegistry(RegistryContext registryContext, UserRealm realm) throws RegistryException {
        this.registryContext = registryContext;
        this.dataSource = registryContext.getDataSource();
        init(realm);
    }

    /**
     * Configures and initiates the JDBC registry with a (new) datasource and a realm. This is
     * useful for changing underlying databases at run-time.
     *
     * @param dataSource the DataSource to use
     * @param realm      Realm to use
     * @throws RegistryException : If something went wrong while init
     */
    public void configure(DataSource dataSource, RegistryRealm realm) throws RegistryException {
        this.dataSource = dataSource;
        init(realm);
    }

    private void init(UserRealm realm) throws RegistryException {

        FileManager.getInstance().init();

        defaultRealm = realm;

        try {
            systemUserRealm = new AuthorizingRealm();
            AuthorizingRealmConfig config =
                    (AuthorizingRealmConfig) systemUserRealm.getRealmConfiguration();
            config.setRealm(defaultRealm);
            config.setAuthenticatedUserName(RegistryConstants.SYSTEM_USER);
            config.setCurrentUserReadable(true);
            config.setCurrentUserEditable(true);
            systemUserRealm.init(config);

        } catch (UserStoreException e) {
            String msg = "Failed to initialize the system realm. " + e.getMessage();
            log.fatal(msg, e);
            throw new RegistryException(msg, e);
        }

        repository = new Repository(dataSource);
        versionRepository = new VersionRepository(dataSource);
        QueryProcessorManager queryProcessorManager =
                new QueryProcessorManager(dataSource, realm, this);

        initializeIndex();

        if (registryContext == null) {
            registryContext = new RegistryContext();
        }
        RegistryContext.setSingleton(registryContext);
        registryContext.setRepository(repository);
        registryContext.setJdbcDir(jdbcDir);
        registryContext.setRamDir(ramDir);

        resourceDAO = new ResourceDAO();

        // check if root is added. if not add it.
        addRootCollection();

        registerBuiltInHandlers();

        basicRegistry =
                new BasicRegistry(registryContext, repository,
                        versionRepository, queryProcessorManager);

        atomicRegistry = new AtomicRegistry(
                basicRegistry, dataSource);
    }

    private void registerBuiltInHandlers() {
        
        HandlerManager handlerManager = registryContext.getHandlerManager();

        SQLQueryHandler sqlQueryHandler = new SQLQueryHandler();
        MediaTypeMatcher sqlMediaTypeMatcher =
                new MediaTypeMatcher(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
        handlerManager.addHandler(
                new String[] {Filter.GET, Filter.PUT}, sqlMediaTypeMatcher, sqlQueryHandler);

        CommentURLHandler commentURLHandler = new CommentURLHandler();
        URLMatcher commentURLMatcher = new URLMatcher();
        commentURLMatcher.setGetPattern(".+;comments:[0-9]+");
        commentURLMatcher.setDeletePattern(".+;comments:[0-9]+");
        handlerManager.addHandler(
                new String[] {Filter.GET, Filter.DELETE} , commentURLMatcher, commentURLHandler);

        CommentCollectionURLHandler commentCollectionURLHandler = new CommentCollectionURLHandler();
        URLMatcher commentCollectionURLMatcher = new URLMatcher();
        commentCollectionURLMatcher.setGetPattern(".+;comments");
        handlerManager.addHandler(new String[] {Filter.GET},
                commentCollectionURLMatcher, commentCollectionURLHandler);

        RatingURLHandler ratingURLHandler = new RatingURLHandler();
        URLMatcher ratingURLMatcher = new URLMatcher();
        ratingURLMatcher.setGetPattern(".+;ratings:.+");
        handlerManager.addHandler(new String[] {Filter.GET}, ratingURLMatcher, ratingURLHandler);

        RatingCollectionURLHandler ratingCollectionURLHandler = new RatingCollectionURLHandler();
        URLMatcher ratingCollectionURLMatcher = new URLMatcher();
        ratingCollectionURLMatcher.setGetPattern(".+;ratings");
        handlerManager.addHandler(
                new String[] {Filter.GET}, ratingCollectionURLMatcher, ratingCollectionURLHandler);

        TagURLHandler tagURLHandler = new TagURLHandler();
        URLMatcher tagURLMatcher = new URLMatcher();
        tagURLMatcher.setGetPattern(".+;.+:.+:.+");
        handlerManager.addHandler(new String[] {Filter.GET}, tagURLMatcher, tagURLHandler);
    }

    private void initializeIndex() throws RegistryException{
        try{
            ResultSet rs = dataSource.getConnection().
                    getMetaData().getTables(null, null, "indexTable", null);
            jdbcDir = new JdbcDirectory(dataSource, DialectFactory.getDialect(dataSource)
                    , "indexTable");
            if(!rs.next()){
                jdbcDir.create();
            }
            ramDir = new RAMDirectory();
            byte [] buffer = new byte [100] ;
            LuceneUtils.copy(jdbcDir, ramDir, buffer);
        }catch(SQLException e){
            throw new RegistryException(e.getMessage());
        }catch(IOException e){
            throw new RegistryException(e.getMessage());
        }

        java.util.Timer timer = new java.util.Timer();
        timer.schedule(new FlushTimer(ramDir,jdbcDir), 0, 300);
    }

    public void beginTransaction() throws RegistryException {

        if (Transaction.isStarted()) {
            String msg = "Failed to start new transaction. Transaction is already " +
                    "started by this thread. Please close started transactions before " +
                    "starting new transaction.";

            log.error(msg);
            throw new RegistryException(msg);
        }

        Connection conn;
        try {
            conn = dataSource.getConnection();
            conn.setAutoCommit(false);
        } catch (SQLException e) {
            String msg = "Failed to start new registry transaction. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        Transaction.setConnection(conn);
    }

    public void rollbackTransaction() throws RegistryException {

        if (!Transaction.isStarted()) {
            String msg = "Failed to rollback transaction. A transaction is not started.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        Connection conn = Transaction.getConnection();
        try {
            conn.rollback();

        } catch (SQLException e) {
            String msg = "Failed to rollback transaction. " + e.getMessage();
            log.error(msg);
            throw new RegistryException(msg);

        } finally {
            endTransaction();
        }
    }

    public void commitTransaction() throws RegistryException {

        if (!Transaction.isStarted()) {
            String msg = "Failed to commit transaction. A transaction is not started.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        Connection conn = Transaction.getConnection();
        try {
            conn.commit();

        } catch (SQLException e) {
            String msg = "Failed to commit transaction. " + e.getMessage();
            log.error(msg);
            throw new RegistryException(msg);

        } finally {
            endTransaction();
        }
    }

    private void endTransaction() throws RegistryException {

        if (Transaction.isStarted()) {
            Connection conn = Transaction.getConnection();
            try {
                conn.close();

            } catch (SQLException e) {
                String msg = "Failed to close transaction. " + e.getMessage();
                log.error(msg);
                throw new RegistryException(msg);

            } finally {
                Transaction.setStarted(false);
                Transaction.setConnection(null);
            }
        }
    }

    public Resource newResource() throws RegistryException {
        return basicRegistry.newResource();
    }

    public Collection newCollection() throws RegistryException {
        return basicRegistry.newCollection();
    }

    public UserRealm getDefaultRealm() {
        return defaultRealm;
    }

    public Resource get(String path) throws RegistryException {

        Resource resource;
        if (Transaction.isStarted()) {
            resource = basicRegistry.get(path);
        } else {
            resource = atomicRegistry.get(path);
        }

        return resource;
    }

    public Collection get(String path, int start, int pageSize) throws RegistryException {
        Collection collection;
        if (Transaction.isStarted()) {
            collection = basicRegistry.get(path, start, pageSize);
        } else {
            collection = atomicRegistry.get(path, start, pageSize);
        }
        return collection;
    }

    public boolean resourceExists(String path) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.resourceExists(path);
        } else {
            return atomicRegistry.resourceExists(path);
        }
    }

    public String put(String suggestedPath, Resource resource) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.put(suggestedPath, resource);
        } else {
            return atomicRegistry.put(suggestedPath, resource);
        }
    }

    public void delete(String path) throws RegistryException {

        if (Transaction.isStarted()) {
            basicRegistry.delete(path);
        } else {
            atomicRegistry.delete(path);
        }
    }

    public String importResource(String suggestedPath, String sourceURL, Resource metadata) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.importResource(suggestedPath, sourceURL, metadata);
        } else {
            return atomicRegistry.importResource(suggestedPath, sourceURL, metadata);
        }
    }

    public String rename(String currentPath, String newPath) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.rename(currentPath, newPath);
        } else {
            return atomicRegistry.rename(currentPath, newPath);
        }
    }

    public String move(String currentPath, String newPath) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.move(currentPath, newPath);
        } else {
            return atomicRegistry.move(currentPath, newPath);
        }
    }

    public String copy(String sourcePath, String targetPath) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.copy(sourcePath, targetPath);
        } else {
            return atomicRegistry.copy(sourcePath, targetPath);
        }
    }

    public void createVersion(String path) throws RegistryException {

        if (Transaction.isStarted()) {
            basicRegistry.createVersion(path);
        } else {
            atomicRegistry.createVersion(path);
        }
    }

    public String[] getVersions(String path) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getVersions(path);
        } else {
            return atomicRegistry.getVersions(path);
        }
    }

    public void restoreVersion(String versionPath) throws RegistryException {

        if (Transaction.isStarted()) {
            basicRegistry.restoreVersion(versionPath);
        } else {
            atomicRegistry.restoreVersion(versionPath);
        }
    }


    ////////////////////////////////////////////////////////
    // Associations
    ////////////////////////////////////////////////////////
    public void addAssociation(String sourcePath, String targetPath, String associationType)
            throws RegistryException {

        if (Transaction.isStarted()) {
            basicRegistry.addAssociation(sourcePath, targetPath, associationType);
        } else {
            atomicRegistry.addAssociation(sourcePath, targetPath, associationType);
        }
    }

    public void removeAssociation(String sourcePath, String targetPath, String associationType)
            throws RegistryException {
        if (Transaction.isStarted()) {
            basicRegistry.removeAssociation(sourcePath, targetPath, associationType);
        } else {
            atomicRegistry.removeAssociation(sourcePath, targetPath, associationType);
        }
    }

    public Association[] getAllAssociations(String resourcePath) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getAllAssociations(resourcePath);
        } else {
            return atomicRegistry.getAllAssociations(resourcePath);
        }
    }

    public Association[] getAssociations(String resourcePath, String associationType)
            throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getAssociations(resourcePath, associationType);
        } else {
            return atomicRegistry.getAssociations(resourcePath, associationType);
        }
    }

    public void applyTag(String resourcePath, String tag) throws RegistryException {

        if (Transaction.isStarted()) {
            basicRegistry.applyTag(resourcePath, tag);
        } else {
            atomicRegistry.applyTag(resourcePath, tag);
        }
    }

    public TaggedResourcePath[] getResourcePathsWithTag(String tag) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getResourcePathsWithTag(tag);
        } else {
            return atomicRegistry.getResourcePathsWithTag(tag);
        }
    }

    public Tag[] getTags(String resourcePath) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getTags(resourcePath);
        } else {
            return atomicRegistry.getTags(resourcePath);
        }
    }

    public void removeTag(String path, String tag) throws RegistryException {

        if (Transaction.isStarted()) {
            basicRegistry.removeTag(path, tag);
        } else {
            atomicRegistry.removeTag(path, tag);
        }
    }

    public String addComment(String resourcePath, Comment comment) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.addComment(resourcePath, comment);
        } else {
            return atomicRegistry.addComment(resourcePath, comment);
        }
    }

    public void editComment(String commentPath, String text) throws RegistryException {

        if (Transaction.isStarted()) {
            basicRegistry.editComment(commentPath, text);
        } else {
            atomicRegistry.editComment(commentPath, text);
        }
    }

    public Comment[] getComments(String resourcePath) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getComments(resourcePath);
        } else {
            return atomicRegistry.getComments(resourcePath);
        }
    }

    public void rateResource(String resourcePath, int rating) throws RegistryException {

        if (Transaction.isStarted()) {
            basicRegistry.rateResource(resourcePath, rating);
        } else {
            atomicRegistry.rateResource(resourcePath, rating);
        }
    }

    public float getAverageRating(String resourcePath) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getAverageRating(resourcePath);
        } else {
            return atomicRegistry.getAverageRating(resourcePath);
        }
    }

    public int getRating(String path, String userName) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getRating(path, userName);
        } else {
            return atomicRegistry.getRating(path, userName);
        }
    }

    public Collection executeQuery(String path, Map parameters) throws RegistryException {

        Collection resultCollection;
        if (Transaction.isStarted()) {
            resultCollection = basicRegistry.executeQuery(path, parameters);
        } else {
            resultCollection = atomicRegistry.executeQuery(path, parameters);
        }

        return resultCollection;
    }

    public LogEntry[] getLogs(String resourcePath, int action, String userName, Date from, Date to, boolean recentFirst) throws RegistryException {
        if (Transaction.isStarted()) {
            return basicRegistry.
                    getLogs(resourcePath, action, userName, from, to, recentFirst);
        } else {
            return atomicRegistry.
                    getLogs(resourcePath, action, userName, from, to, recentFirst);
        }
    }


    public LogEntryCollection getLogCollection(String resourcePath,
                                               int action,
                                               String userName,
                                               Date from,
                                               Date to,
                                               boolean recentFirst) throws RegistryException {
        LogEntryCollection logEntryCollection ;
        if (Transaction.isStarted()) {
            logEntryCollection = basicRegistry.
                    getLogCollection(resourcePath, action, userName, from, to, recentFirst);
        } else {
            logEntryCollection = atomicRegistry.
                    getLogCollection(resourcePath, action, userName, from, to, recentFirst);
        }
        logEntryCollection.setDataSource(dataSource);
        logEntryCollection.setResourcePath(resourcePath);
        logEntryCollection.setAction(action);
        logEntryCollection.setUserName(userName);
        logEntryCollection.setFrom(from);
        logEntryCollection.setTo(to);
        logEntryCollection.setRecentFirst(recentFirst);
        return logEntryCollection;

    }

    public String[] getAvailableAspects() {
        return registryContext.getAspectNames();
    }

    public void associateAspect(String resourcePath, String aspect) throws RegistryException {
        if (Transaction.isStarted()) {
            basicRegistry.associateAspect(resourcePath, aspect);
        } else {
            atomicRegistry.associateAspect(resourcePath, aspect);
        }
    }

    public void invokeAspect(String resourcePath, String aspectName, String action) throws RegistryException {
        if (Transaction.isStarted()) {
            basicRegistry.invokeAspect(resourcePath, aspectName, action);
        } else {
            atomicRegistry.invokeAspect(resourcePath, aspectName, action);
        }
    }

    public String[] getAspectActions(String resourcePath, String aspectName) throws RegistryException {

        if (Transaction.isStarted()) {
            return basicRegistry.getAspectActions(resourcePath, aspectName);
        } else {
            return atomicRegistry.getAspectActions(resourcePath, aspectName);
        }
    }

    private void addRootCollection() throws RegistryException {

        CurrentSession.setUser(RegistryConstants.SYSTEM_USER);
        CurrentSession.setRealm(systemUserRealm);

        try {

            beginTransaction();

            if (!resourceDAO.resourceExists(RegistryConstants.ROOT_PATH)) {

                CollectionImpl root = new CollectionImpl();
                resourceDAO.addRoot(RegistryConstants.ROOT_PATH, root);

                AuthorizationUtils.setRootAuthorizations(root.getId(), defaultRealm);
            }

            commitTransaction();

        } catch (Exception e) {

            String msg = "Failed to add the root collection to the registry.";
            log.fatal(msg, e);

            rollbackTransaction();

            throw new RegistryException(msg, e);
        }

        CurrentSession.removeUser();
        CurrentSession.removeRealm();

    }

    public Collection searchContent(String keywords) throws RegistryException{
        if (Transaction.isStarted()) {
            return basicRegistry.searchContent(keywords);
        } else {
            return atomicRegistry.searchContent(keywords);
        }
    }

    public RegistryContext getRegistryContext() {
        return registryContext;
    }


}
