/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.dao.*;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.session.CurrentSession;
import org.wso2.registry.utils.AuthorizationUtils;
import org.wso2.registry.utils.RegistryUtils;
import org.wso2.registry.utils.UUIDGenerator;

import javax.sql.DataSource;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Iterator;
import java.util.List;

/**
 * Encapsulates the retrieving, storing, modifing and deleting of resources. This class only deals
 * with the current versions of resources and it is unware of any versioning or snapshot activity.
 * Only the current version related tables are accessed and updated from the methods of this class.
 */
public class Repository {

    public static final Log log = LogFactory.getLog(Repository.class);

    private ResourceDAO resourceDAO;
    private ResourceVersionDAO resourceVersionDAO;
    private AssociationDAO associationDAO;
    private LogsDAO logsDAO;

    /**
     * Datasource is set on all resources get from the repository.
     */
    private DataSource dataSource;

    public Repository(DataSource dataSource) {
        this.dataSource = dataSource;
        resourceDAO = new ResourceDAO();
        resourceVersionDAO = new ResourceVersionDAO();
        associationDAO = new AssociationDAO();
        logsDAO = new LogsDAO();
    }

    /**
     * Checks if a pure resource exists in the given path.
     *
     * @param path Path of a possible pure resource.
     * @return ture if a resource exists in the given path. false otherwise.
     */
    public boolean resourceExists(String path) throws RegistryException {

        String purePath = RegistryUtils.getPureResourcePath(path);

        return resourceDAO.resourceExists(purePath);
    }

    /**
     * Gets the pure resource referred by the given path. If purePath referes to a specific version of
     * the resource, this gives the refered version. If path does not contain version information,
     * this gives the current version of the resource.
     *
     * @param path Path of a "pure" resource. Path should map to an actual resource stored
     *             in the database. Paths refering to virtual resource are not handled (e.g. /c1/r1;comments).
     * @return Resource refered by the given path. Resource can be a file or a collection.
     */
    public Resource get(String path) throws RegistryException {

        String purePath = RegistryUtils.getPureResourcePath(path);

        String resourceID = ResourceDAO.getResourceID(purePath);
        if (resourceID == null) {
            return null;
        }

        if (!AuthorizationUtils.authorize(resourceID, ActionConstants.GET)) {
            String msg = "User " + CurrentSession.getUser() + " is not authorized to " +
                    "read the resource " + purePath + ".";
            log.error(msg);
            throw new AuthorizationFailedException(msg);
        }

        Resource resource = resourceDAO.get(purePath);
        ((ResourceImpl) resource).setDataSource(dataSource);

        return resource;
    }

    public Collection get(String path, int start, int pageLen) throws RegistryException {
        String purePath = RegistryUtils.getPureResourcePath(path);

        String resourceID = ResourceDAO.getResourceID(purePath);
        if (resourceID == null) {
            return null;
        }

        if (!AuthorizationUtils.authorize(resourceID, ActionConstants.GET)) {
            String msg = "User " + CurrentSession.getUser() + " is not authorized to " +
                    "read the resource " + purePath + ".";
            log.error(msg);
            throw new AuthorizationFailedException(msg);
        }

        CollectionImpl resource = resourceDAO.get(purePath, start, pageLen);
        resource.setDataSource(dataSource);

        return resource;
    }

    /**
     * Adds or updates the resource in the given path with the given resource. Put is executed if
     * the current user has authorization to do so. Below is the method of evaluating
     * authorizations.
     * <p/>
     * <ul>
     * <li> user should have WRITE permissions for the resource, if he wants to update the
     * resource.</li>
     * <li>user should habe WRITE permissions for the parent of the new resource, if he wants to
     * add a new resource.</li>
     * </ul>
     *
     * @param path     Path of the resource to be added or updated. Path should only refer to current
     *                 version a pure resource. i.e. Path should not contain any version information as it is
     *                 impossible to add or update old versions.
     * @param resource Resource to be added or updated.
     */
    public void put(String path, Resource resource) throws RegistryException {

        String purePath = RegistryUtils.getPureResourcePath(path);

        String resourceID;
        if ((resourceID = ResourceDAO.getResourceID(purePath)) != null) {

            // resource already exists. try to update the resource
            update(resourceID, purePath, resource);

            if (((ResourceImpl) resource).isVersionableChange()) {
                resourceDAO.markDirty(resourceID);
            }

        } else {

            // resource does not exists. add the resource.
            add(purePath, resource);

            String parentPath = RegistryUtils.getParentPath(purePath);
            resourceDAO.markDirtyByPath(parentPath);
        }

        // all the ascendant collections of this resource gets modified. that is, if snapshot are
        // created for those collections, all such snapshots get invalid.
        invalidateParentSnapshots(path);
    }

    /**
     * Creates a resource with the content imported from the source URL and metadata extracted from
     * the given metadata resource instance. Then the created resource is put to the registry using
     * the Repository.put() method.
     *
     * @param path      Path to put the resource
     * @param sourceURL URL to import resource content
     * @param metadata  Metadata for the new resource is extracted from this metadata resource
     * @return Actual path where the new resource is stored in the registry
     * @throws RegistryException for all exceptional scenarios
     */
    public String importResource(String path, String sourceURL, Resource metadata)
            throws RegistryException {

        String purePath = RegistryUtils.getPureResourcePath(path);

        URL url;
        try {
            url = new URL(sourceURL);
        } catch (MalformedURLException e) {
            String msg = "Given source URL is not valid.";
            throw new RegistryException(msg, e);
        }

        try {
            URLConnection uc = url.openConnection();
            InputStream in = uc.getInputStream();
//            resource.setPath(purePath);
            String mediaType = metadata.getMediaType();
            if (mediaType == null)
                mediaType = uc.getContentType();
            metadata.setMediaType(mediaType);
            metadata.setDescription(metadata.getDescription());
            metadata.setContentStream(in);
            put(purePath, metadata);

        } catch (IOException e) {

            String msg = "Could not read from the given URL: " + sourceURL;
            throw new RegistryException(msg, e);
        }

        String parentPath = RegistryUtils.getParentPath(purePath);
        resourceDAO.markDirtyByPath(parentPath);

        return purePath;
    }

    /**
     * Deletes the pure resource refered by the path.
     *
     * @param path path Path of the resource deleted. Path should only refer to current
     *             version a pure resource. i.e. Path should not contain any version information as it is
     *             impossible to delete old versions.
     */
    public void delete(String path) throws RegistryException {

        path = RegistryUtils.getPureResourcePath(path);

        String resourceID = ResourceDAO.getResourceID(path);
        if (resourceID == null) {
            String msg = "Failed to delete resource " + path + ". Resource does not exist.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        if (!AuthorizationUtils.authorize(resourceID, ActionConstants.DELETE)) {
            String msg = "User " + CurrentSession.getUser() + " is not authorized to " +
                    "delete the resource " + path + ".";
            log.error(msg);
            throw new AuthorizationFailedException(msg);
        }

        String parentPath = RegistryUtils.getParentPath(path);

        // parent collection gets dirty when a resource is deleted from it
        resourceDAO.markDirtyByPath(parentPath);

        resourceDAO.deleteSubtree(resourceID);

        invalidateParentSnapshots(path);
    }

    /**
     * Renames the resource at oldPath with the given newName. If oldPath is
     * "/projects/esb/config.xml" and newName is "synapse.xml" then the new path of the resource
     * will be "/projects/esb/synapse.xml". If the renamed resource is a collection, paths of
     * all its desendants will also be changed.
     *
     * @param oldResourcePath Path of the resource to be renamed.
     * @param newName New name for the resource. Name should not contain "/".
     * @return New path of the resource.
     * @throws RegistryException
     */
    public String rename(ResourcePath oldResourcePath, String newName) throws RegistryException {

        if (!oldResourcePath.isCurrentVersion()) {
            String msg = "Failed to rename the resource " + oldResourcePath +
                    ". Given path refers to an archived version of the resource.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        String oldPath = RegistryUtils.getPureResourcePath(oldResourcePath.getPath());

        if (!newName.startsWith(RegistryConstants.PATH_SEPARATOR)) {
            // relative URL
            String oldDir =
                    oldPath.substring(0, oldPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) + 1);
            newName = oldDir + newName;
        }

        String resourceID = ResourceDAO.getResourceID(oldPath);
        resourceDAO.setPath(resourceID, newName);

        // we don't support browsing previous versions (snapshots) of resources after renaming
        // them currently
        resourceVersionDAO.removeSnapshotsOfResource(resourceID);

        List<String> childIDs = resourceDAO.getChildIDs(resourceID);
        Iterator<String> iChild = childIDs.iterator();
        while (iChild.hasNext()) {
            String childID = iChild.next();
            renameChildren(childID, newName);
        }
         AssociationDAO.copyAssociations(oldPath ,newName);

        return newName;
    }

    /**
     * Moves the resource at oldPath to the newPath. If the resource is a collection, all its
     * descendant resources will also be moved.
     *
     * @param oldResourcePath Path of a existing resource
     * @param newPath New path of the resource
     * @return the actual new path
     * @throws RegistryException
     */
    public String move(ResourcePath oldResourcePath, String newPath) throws RegistryException {

         if (!oldResourcePath.isCurrentVersion()) {
            String msg = "Failed to copy the resource " + oldResourcePath +
                    ". Given path refers to an archived version of the resource.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        // we are implementing move as copying the resource to the new path and deleting the old
        // resource. therefore, the moved resource will be threated as a new resource (i.e without
        // a version hostory. but the resource at old path (which is deleted) has its version
        // hostory. so it is possible to restore the parent collection of old path and get old
        // versions of that resource.

        String oldPath = oldResourcePath.getPath();
        copy(new ResourcePath(oldPath), new ResourcePath(newPath));

        // once the resource is moved all associations with the old resource has to be reassociated
        // with the new resource
        associationDAO.replaceAssociations(oldPath, newPath);
        
        delete(oldPath);
        return newPath;
    }

    public String copy(ResourcePath sourceResourcePath, ResourcePath targetResourcePath)
            throws RegistryException {

         if (!sourceResourcePath.isCurrentVersion() || !targetResourcePath.isCurrentVersion()) {
            String msg = "Failed to copy the resource " + sourceResourcePath + " to path " +
                    targetResourcePath + ". Both paths should refer to current versions.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        String sourcePath = sourceResourcePath.getPath();
        String targetPath = targetResourcePath.getPath();
        
        sourcePath = RegistryUtils.getPureResourcePath(sourcePath);
        targetPath = RegistryUtils.getPureResourcePath(targetPath);

        ResourceImpl resource = (ResourceImpl) get(sourcePath);
        final String newID = UUIDGenerator.generateUUID();
        resource.setId(newID);
        put(targetPath, resource);

        String resourceID = ResourceDAO.getResourceID(sourcePath);
        List<String> childIDs = resourceDAO.getChildIDs(resourceID);
        Iterator<String> iChild = childIDs.iterator();
        while (iChild.hasNext()) {
            String childID = iChild.next();
            copy(sourcePath, targetPath, childID);
        }

        CommentsDAO.copyComments(sourcePath, newID);
        TagsDAO.copyTags(resourceID, newID);
        RatingsDAO.copyRatings(resourceID, newID);
        AssociationDAO.copyAssociations(sourcePath ,targetPath);
        return targetPath;
    }

    private void copy(String sourcePath, String targetPath, String resourceID)
            throws RegistryException {

        String resourcePath = resourceDAO.getResourcePath(resourceID);
        ResourceImpl resource = (ResourceImpl) get(resourcePath);
        String newId = UUIDGenerator.generateUUID();
        resource.setId(newId);

        String targetPathForThisResource = targetPath + resourcePath.substring(sourcePath.length());
        put(targetPathForThisResource, resource);

        List<String> childIDs = resourceDAO.getChildIDs(resourceID);
        Iterator<String> iChild = childIDs.iterator();
        while (iChild.hasNext()) {
            String childID = iChild.next();
            copy(sourcePath, targetPath, childID);
        }

        CommentsDAO.copyComments(resourcePath, newId);
        TagsDAO.copyTags(resourceID, newId);
        RatingsDAO.copyRatings(resourceID, newId);
        AssociationDAO.copyAssociations(resourcePath ,targetPathForThisResource);
    }

    private void add(String path, Resource resource) throws RegistryException {

        // first add all non-existing parent collections. note that whether the user has
        // permission to add this resource depends on the permissions of the nearest ascendant.

        String parentPath = RegistryUtils.getParentPath(path);

        // first, let's check if there is a parent for this resource. we do this to speed up the
        // common use case where new resources are added to an existing parent.
        String parentID = ResourceDAO.getResourceID(parentPath);
        if (parentID == null) {

            // no parent exists for this the new resource. so we have to add all missing
            // ancestor collections. this operation may be bit slow. but we ignore that as this
            // is not very frequent.
            addParents(path);
            parentID = ResourceDAO.getResourceID(parentPath);
        }

        if (!AuthorizationUtils.authorize(parentID, ActionConstants.PUT)) {
            String msg = "Failed to add new resource " + path + ". User " +
                    CurrentSession.getUser() + " is not authorized to update " +
                    "the parent collection " + parentPath + ".";
            log.error(msg);
            throw new AuthorizationFailedException(msg);
        }

        // then add the new resource/collection
        logsDAO.addLog(path, CurrentSession.getUser(), LogEntry.ADD, null);
        resourceDAO.add(path, parentID, (ResourceImpl) resource);

        // set authorizations for the newly added resource
        AuthorizationUtils.copyAuthorizations(parentID, resource.getId());
    }

    private void invalidateParentSnapshots(String path) throws RegistryException {

        if (RegistryConstants.ROOT_PATH.equals(path)) {
            return;
        }

        String parentPath = RegistryUtils.getParentPath(path);

        resourceDAO.invalidateCurrentSnapshot(RegistryConstants.ROOT_PATH);
        
        if (RegistryConstants.ROOT_PATH.equals(parentPath)) {
            return;
        }

        String[] parts = parentPath.split(RegistryConstants.PATH_SEPARATOR);

        String currentPath = RegistryConstants.ROOT_PATH + parts[1];
        resourceDAO.invalidateCurrentSnapshot(currentPath);

        for (int i = 2; i < parts.length; i++) {

            currentPath = currentPath + RegistryConstants.PATH_SEPARATOR + parts[i];
            resourceDAO.invalidateCurrentSnapshot(currentPath);
        }
    }

    private void update(String resourceID, String path, Resource resource)
            throws RegistryException {

        // resources are referred exclusively by the resource ID in the database. API users may
        // create new resources and put them to existing paths. In that case, we have to set the
        // ID of the existing resource, before giving it the database layer.
        ((ResourceImpl) resource).setId(resourceID);

        if (!AuthorizationUtils.authorize(resourceID, ActionConstants.PUT)) {
            String msg = "Resource update failed. User " + CurrentSession.getUser() +
                    " is not authorized to update the resource " + path + ".";
            log.error(msg);
            throw new AuthorizationFailedException(msg);
        }

        logsDAO.addLog(path, CurrentSession.getUser(), LogEntry.UPDATE, null);
        resourceDAO.update(resourceID, (ResourceImpl) resource);

    }

    /**
     * Adds all non-existent collections of the given path starting from the root. Note that we
     * are not using recursive method call here to avoid possible stack overflaws due to deep
     * trees.
     * <p/>
     * Start from the root and check each ancestor collection exists. If we find any ancestor
     * "resource" we can give this up, as resources cannot contain other collcetions or resources.
     * When we find the first missing collection, we have to check whether the user has PUT
     * permission for the parent of that collection. If user does, add a new collection.
     * <p/>
     * Now we don't have to worry about permissions anymore. Because authorizations are copied
     * from parent to child, user can add all other required resources.
     *
     * @param path Path of which all non-existent collections are added.
     * @throws RegistryException If any ancestor of the given path is a resource.
     */
    private void addParents(String path) throws RegistryException {

        String parentPath = RegistryUtils.getParentPath(path);

        String[] parts = parentPath.split(RegistryConstants.PATH_SEPARATOR);

        String idOfLastExistingParent = null;
        String currentPath = RegistryConstants.ROOT_PATH + parts[1];
        for (int i = 2; i < parts.length + 1; i++) {

            if (idOfLastExistingParent == null) {

                int resourceType = resourceDAO.getResourceType(currentPath);
                if (resourceType == ResourceDAO.NULL) {

                    String pathOfLastParent = RegistryUtils.getParentPath(currentPath);
                    idOfLastExistingParent = ResourceDAO.getResourceID(pathOfLastParent);

                    if (!AuthorizationUtils.authorize(
                            idOfLastExistingParent, ActionConstants.PUT)) {
                        String msg = "Failed to add new resource. User " +
                                CurrentSession.getUser() + " does not have authorization to " +
                                "update the collection " + pathOfLastParent + ".";
                        log.error(msg);
                        throw new RegistryException(msg);
                    }

                    CollectionImpl collection = new CollectionImpl();
                    logsDAO.addLog(currentPath, CurrentSession.getUser(), LogEntry.ADD, null);
                    resourceDAO.add(currentPath, idOfLastExistingParent, collection);

                    AuthorizationUtils.copyAuthorizations(idOfLastExistingParent, collection.getId());

                } else if (resourceType == ResourceDAO.RESOURCE) {

                    String msg = "Failed to add new resource. " + currentPath + " is a non-collection " +
                            "ancestor located in the given new resource path.";
                    log.error(msg);
                    throw new RegistryException(msg);
                }

            } else {

                String currentParentPath = RegistryUtils.getParentPath(currentPath);
                String currentParentID = ResourceDAO.getResourceID(currentParentPath);

                CollectionImpl collection = new CollectionImpl();
                logsDAO.addLog(currentPath, CurrentSession.getUser(), LogEntry.ADD, null);
                resourceDAO.add(currentPath, currentParentID, collection);

                AuthorizationUtils.copyAuthorizations(idOfLastExistingParent, collection.getId());

            }

            if (i < parts.length) {
                currentPath = currentPath + RegistryConstants.PATH_SEPARATOR + parts[i];
            }
        }
    }

    private void renameChildren(String resourceID, String newParentPath) throws RegistryException {

        String resourcePath = resourceDAO.getResourcePath(resourceID);
        String newPath = newParentPath + resourcePath.substring(
                resourcePath.lastIndexOf(RegistryConstants.PATH_SEPARATOR), resourcePath.length());

        resourceDAO.setPath(resourceID, newPath);

        // we don't support browsing previous versions (snapshots) of resources after renaming
        // them currently
        resourceVersionDAO.removeSnapshotsOfResource(resourceID);

        List<String> childIDs = resourceDAO.getChildIDs(resourceID);
        Iterator<String> iChild = childIDs.iterator();
        while (iChild.hasNext()) {
            String childID = iChild.next();
            renameChildren(childID, newPath);
        }
         AssociationDAO.copyAssociations(resourcePath ,newPath);
    }
}
