/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Association;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.utils.Transaction;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

public class AssociationDAO {

    private static final Log log = LogFactory.getLog(AssociationDAO.class);

    public static void addAssociation(String sourcePath,
                               String targetPath,
                               String associationType) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String propSQL =
                    "INSERT INTO REG_ASSOCIATION (SOURCEPATH, TARGETPATH, ASSOCIATION_TYPE) VALUES (?,?,?)";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, sourcePath);
            ps.setString(2, targetPath);
            ps.setString(3, associationType);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to add association between resources " +
                    sourcePath + " and " + targetPath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void removeAssociation(String sourcePath,
                                  String targetPath,
                                  String associationType) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String propSQL =
                    "DELETE FROM REG_ASSOCIATION WHERE SOURCEPATH=? AND TARGETPATH=? AND ASSOCIATION_TYPE=?";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, sourcePath);
            ps.setString(2, targetPath);
            ps.setString(3, associationType);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {
            String msg = "Failed to remove association between resources " +
                    sourcePath + " and " + targetPath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }


    public Association[] getAllAssociations(String resourcePath) throws RegistryException {

        Connection conn = Transaction.getConnection();

        ArrayList arrayList;
        try {
            String propSQL = "SELECT * FROM REG_ASSOCIATION WHERE SOURCEPATH=? OR TARGETPATH=?";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, resourcePath);
            ps.setString(2, resourcePath);
            ResultSet result = ps.executeQuery();
            arrayList = new ArrayList();

            while (result.next()) {
                Association association = new Association();
                association.setSourcePath(result.getString("SOURCEPATH"));
                association.setDestinationPath(result.getString("TARGETPATH"));
                association.setAssociationType(result.getString("ASSOCIATION_TYPE"));
                arrayList.add(association);
            }
            ps.close();
        } catch (SQLException e) {

            String msg = "Failed to get all associations of resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        Association associations[] = new Association[arrayList.size()];
        for (int i = 0; i < arrayList.size(); i++) {
            associations[i] = (Association) arrayList.get(i);
        }
        return associations;
    }

    public Association[] getAllAssociationsForType(String resourcePath, String associationType)
            throws RegistryException {
        Connection conn = Transaction.getConnection();

        ArrayList arrayList;
        try {
            String propSQL = "SELECT SOURCEPATH, TARGETPATH FROM REG_ASSOCIATION " +
                             "WHERE (SOURCEPATH=? OR TARGETPATH=?) AND ASSOCIATION_TYPE=?";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, resourcePath);
            ps.setString(2, resourcePath);
            ps.setString(3, associationType);
            ResultSet result = ps.executeQuery();
            arrayList = new ArrayList();

            while (result.next()) {
                Association association = new Association();
                association.setSourcePath(result.getString("SOURCEPATH"));
                association.setDestinationPath(result.getString("TARGETPATH"));
                association.setAssociationType(associationType);
                arrayList.add(association);
            }
            ps.close();
        } catch (SQLException e) {

            String msg = "Failed to get associations of type " +
                    associationType + " for resource " + resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        Association associations[] = new Association[arrayList.size()];
        for (int i = 0; i < arrayList.size(); i++) {
            associations[i] = (Association) arrayList.get(i);
        }
        return associations;
    }

    public void replaceAssociations(String oldPath, String newPath) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql2 = "UPDATE REG_ASSOCIATION SET TARGETPATH=? WHERE TARGETPATH=?";

            PreparedStatement ps2 = conn.prepareStatement(sql2);
            ps2.setString(1, newPath);
            ps2.setString(2, oldPath);
            ps2.executeUpdate();
            ps2.close();

        } catch (SQLException e) {

            String msg = "Failed to replace the associations of " + oldPath +
                    " by reassociating them to " + newPath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void removeAllAssociations(String resourcePath) throws RegistryException {

        Connection conn = Transaction.getConnection();

        String sql = "DELETE FROM REG_ASSOCIATION WHERE SOURCEPATH=? OR TARGETPATH=?";

        try {
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourcePath);
            ps.setString(2, resourcePath);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to remove associations of resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public static void copyAssociations(String fromPath ,String toPath ) throws RegistryException {
        try {
            Connection conn = Transaction.getConnection();
            String propSQL = "SELECT * FROM REG_ASSOCIATION WHERE SOURCEPATH=?";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, fromPath);
            ResultSet result = ps.executeQuery();
            ArrayList arrayList = new ArrayList();

            while (result.next()) {
                Association association = new Association();
                association.setSourcePath(toPath);
                association.setDestinationPath(result.getString("TARGETPATH"));
                association.setAssociationType(result.getString("ASSOCIATION_TYPE"));
                arrayList.add(association);
            }

            for (int i = 0; i < arrayList.size(); i++) {
                Association association = (Association) arrayList.get(i);
                addAssociation(toPath,association.getDestinationPath() , association.getAssociationType());
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }
}
