/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.registry.Collection;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.dataobjects.ResourceDO;
import org.wso2.registry.jdbc.filecache.FileManager;
import org.wso2.registry.jdbc.utils.Transaction;
import org.wso2.registry.session.CurrentSession;
import org.wso2.registry.utils.AuthorizationUtils;
import org.wso2.registry.utils.UUIDGenerator;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.util.Date;

public class ResourceDAO {

    private static final Log log = LogFactory.getLog(ResourceDAO.class);

    public static final int NULL = -1;
    public static final int RESOURCE = 0;
    public static final int COLLECTION = 1;

    private AssociationDAO associationDAO;

    public ResourceDAO() {
        associationDAO = new AssociationDAO();
    }

    /**
     * Returns the resource ID (RID) of the resource refered by the given path. If a resource does
     * not exists in the given path, it returns null.
     *
     * @param path Pure path of the resource
     * @return RID of resource exists. null if the resource does not exists.
     * @throws org.wso2.registry.exceptions.RegistryException something went wrong
     */
    public static String getResourceID(String path) throws RegistryException {

        Connection conn = Transaction.getConnection();

        String resourceID = null;
        try {
            String sql = "SELECT RID FROM REG_RESOURCE WHERE PATH=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, path);

            ResultSet result = ps.executeQuery();

            if (result.next()) {
                resourceID = result.getString("RID");
            }

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to get ID of the resource at path " + path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return resourceID;
    }

    /**
     * Returns the resource ID (RID) of the resource refered by the given path. If a resource does
     * not exists in the given path, it returns null. This method is used when the registry does
     * participate in an ongoing transaction.
     *
     * @param path Pure path of the resource
     * @param conn Database connection for accessing the REG_RESOURCE table
     * @return RID of resource exists. null if the resource does not exists.
     * @throws java.sql.SQLException something went wrong
     */
    public String getResourceID(String path, Connection conn) throws SQLException {

        String resourceID = null;

        String sql = "SELECT RID FROM REG_RESOURCE WHERE PATH=?";

        PreparedStatement ps = conn.prepareStatement(sql);
        ps.setString(1, path);

        ResultSet result = ps.executeQuery();

        if (result.next()) {
            resourceID = result.getString("RID");
        }

        ps.close();

        return resourceID;
    }

    public String getResourcePath(String resourceID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        String resourcePath = null;
        try {
            String sql = "SELECT PATH FROM REG_RESOURCE WHERE RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResultSet result = ps.executeQuery();

            if (result.next()) {
                resourcePath = result.getString("PATH");
            }

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to get path of the resource with ID " + resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return resourcePath;
    }

    /**
     * Returns the type of the resource refered by the given path.
     *
     * @param path Pure path of a resource
     * @return  ResourceDAO.RESOURCE: path referes to a resource
     *          ResourceDAO.COLLECTION: path referes to a collection
     *          ResourceDAO.NULL: resource does not exist
     * @throws org.wso2.registry.exceptions.RegistryException something went wrong
     */
    public int getResourceType(String path) throws RegistryException {

        Connection conn = Transaction.getConnection();

        int resourceType;
        try {
            String sql = "SELECT COLLECTION FROM REG_RESOURCE WHERE PATH=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, path);

            ResultSet result = ps.executeQuery();

            resourceType = ResourceDAO.NULL;
            if (result.next()) {
                int collection = result.getInt("COLLECTION");

                if (collection == 0) {
                    resourceType = ResourceDAO.RESOURCE;
                } else {
                    resourceType = ResourceDAO.COLLECTION;
                }
            }

            ps.close();

        } catch (SQLException e) {

            String msg = "Faile to get the resource type of the resource " +
                    path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return resourceType;
    }

    public boolean resourceExists(String path) throws RegistryException {

        Connection conn = Transaction.getConnection();

        boolean exists;
        try {
            String sql = "SELECT PATH FROM REG_RESOURCE WHERE PATH=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, path);

            ResultSet result = ps.executeQuery();

            exists = result.next();

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to check the existence of the resource " +
                    path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return exists;
    }

    public boolean resourceExistsWithID(String resourceID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        boolean exists;
        try {
            String sql = "SELECT PATH FROM REG_RESOURCE WHERE RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResultSet result = ps.executeQuery();

            exists = result.next();

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to check the existence of the resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return exists;
    }

    public void attachChild(String parentID, String childID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "INSERT INTO REG_DEPENDENCY (PARENT_RID, CHILD_RID) VALUES (?, ?)";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, parentID);
            ps.setString(2, childID);

            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {
            String msg = "Failed to add child resource  " + childID +
                    " to parent resource " + parentID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void detachChild(String childID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "DELETE FROM REG_DEPENDENCY WHERE CHILD_RID=?";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, childID);

            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {
            String msg = "Failed to remove the child resource with ID " +
                    childID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    /**
     * Returns the resource in the given path filled with metadata and access to the content.
     * If a resource does not exist in the given path, null is returned.
     *
     * @param path Path of the resource.
     *
     * @return ResourceImpl filled with resource metadata and access to the resource content.
     * @throws org.wso2.registry.exceptions.RegistryException something went wrong
     */
    public ResourceImpl get(String path) throws RegistryException {

        ResourceImpl resourceImpl = getResource(path);

        //if (resourceImpl instanceof Collection) {
        //    fillChildren((CollectionImpl)resourceImpl, -1, -1);
        //}

        fillResourceProperties(resourceImpl);

        return resourceImpl;
    }

    public CollectionImpl get(String path, int start, int pageLen) throws RegistryException {
        CollectionImpl collection = (CollectionImpl)getResource(path);
        fillChildren(collection, start, pageLen);
        fillResourceProperties(collection);
        return collection;
    }

    public void fillResourceProperties(ResourceImpl resourceImpl) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String propSQL = "SELECT NAME, PROPERTY_VALUE FROM REG_PROPERTY WHERE RID=?";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, resourceImpl.getId());
            ResultSet results = ps.executeQuery();

            while (results.next()) {
                String name = results.getString("NAME");
                String value = results.getString("PROPERTY_VALUE");
                resourceImpl.addProperty(name, value);
            }

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to add properties to the resource " +
                    resourceImpl.getPath() + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        resourceImpl.setPropertiesModified(false);
    }

    public void addRoot(String path, ResourceImpl resourceImpl)
            throws RegistryException {

        addResource(path, resourceImpl);

        addProperties(resourceImpl);
    }

    public void add(String path, String parentID, ResourceImpl resourceImpl)
            throws RegistryException {

        if (!(resourceImpl instanceof Collection)) {
            addContent(resourceImpl);
        }

        addResource(path, resourceImpl);

        addDependency(parentID, resourceImpl.getId());

        addProperties(resourceImpl);
    }

    public void update(String resourceID, ResourceImpl resourceImpl) throws RegistryException {


        if (resourceImpl.isContentModified()) {
            addContent(resourceImpl);
        }

        updateResource(resourceID, resourceImpl);

        if (resourceImpl.isPropertiesModified()) {
            updateProperties(resourceImpl);
        }
    }

    public int getEquivalentVersion(String resourceID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        int equivalentVersion = -2;
        try {
            String sql = "SELECT EQUIVALENT_VERSION FROM REG_RESOURCE WHERE RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResultSet result = ps.executeQuery();
            if (result.next()) {
                equivalentVersion = result.getInt("EQUIVALENT_VERSION");
            }

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to get the equivalen version of resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return equivalentVersion;
    }

    public void markDirty(String resourceID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "UPDATE REG_RESOURCE SET EQUIVALENT_VERSION=?, ASSOCIATED_SNAPSHOT_ID=? " +
                    "WHERE RID=?";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setLong(1, -1);
            ps.setLong(2, -1);
            ps.setString(3, resourceID);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to update the equivalent version of the resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void delete(String resourceID) throws RegistryException {

        String contentID = getContentID(resourceID);

        deleteResource(resourceID);

        if (contentID != null) {
            // delete content checks whether the content usage is zero before deleting the content
            deleteContent(contentID);
        }
    }

    private String getContentID(String resourceID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "SELECT CONTENT_ID FROM REG_RESOURCE WHERE RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            String contentID = null;
            ResultSet result = ps.executeQuery();
            if (result.next()) {
                contentID = result.getString("CONTENT_ID");
            }
            ps.close();

            return contentID;

        } catch (SQLException e) {

            String msg = "Failed to get content ID of the resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void deleteSubtree(String resourceID) throws RegistryException {

        List <String> childIDs = getChildIDs(resourceID);

        Iterator <String> iChildIDs = childIDs.iterator();
        while (iChildIDs.hasNext()) {
            String childID = iChildIDs.next();
            deleteSubtree(childID);
        }

        String resourcePath = getResourcePath(resourceID);
        associationDAO.removeAllAssociations(resourcePath);

        delete(resourceID);
    }

    public List <String> getChildIDs(String resourceID) throws RegistryException {

        List <String> childIDs = new ArrayList();

        Connection conn = Transaction.getConnection();

        try {
            String sql = "SELECT CHILD_RID FROM REG_DEPENDENCY WHERE PARENT_RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResultSet results = ps.executeQuery();
            while (results.next()) {
                childIDs.add(results.getString("CHILD_RID"));
            }
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to get child resource IDs of resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return childIDs;
    }

    public int getChildCount(String resourceID, Connection conn) throws RegistryException {

        try {

            int childCount = 0;

            String sql = "SELECT CHILD_RID FROM REG_DEPENDENCY WHERE PARENT_RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResultSet results = ps.executeQuery();
            while (results.next()) {

                String childID = results.getString("CHILD_RID");

                if (AuthorizationUtils.authorize(childID, ActionConstants.GET)) {
                    childCount++;
                }
            }
            ps.close();

            return childCount;

        } catch (SQLException e) {

            String msg = "Failed to get child count of resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public List <String> getChildPaths(String resourceID, Connection conn)
            throws RegistryException {

        List <String> childPaths = new ArrayList <String> ();

        try {
            String sql = "SELECT R.PATH, R.RID FROM REG_RESOURCE R, REG_DEPENDENCY D WHERE " +
                    "R.RID=D.CHILD_RID AND PARENT_RID=? ORDER BY R.PATH";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResultSet results = ps.executeQuery();
            while (results.next()) {
                String childID = results.getString("RID");
                if (AuthorizationUtils.authorize(childID, ActionConstants.GET)) {
                    childPaths.add(results.getString("PATH"));
                }
            }
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to get child resource IDs of resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return childPaths;
    }

    public List <String> getChildPaths(String resourceID, int start, int pageLen, Connection conn)
            throws RegistryException {

        try {

            String sql = "SELECT R.RID, R.PATH  FROM REG_RESOURCE R, REG_DEPENDENCY D WHERE D.PARENT_RID=?" +
                    " AND D.CHILD_RID=R.RID ORDER BY R.PATH";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);
            ResultSet results = ps.executeQuery();

            List<String> childPathList = new ArrayList <String> ();
            int current = 0;
            int end = start + pageLen;
            while (results.next() && current <= end) {
                String childID = results.getString("RID");
                if (AuthorizationUtils.authorize(childID, ActionConstants.GET)) {
                    if (current >= start && (pageLen == -1 || current < end))
                        childPathList.add(results.getString("PATH"));
                    current++;
                }
            }

            if (current < start) {
                throw new RegistryException("Didn't have enough results to start at #" + start);
            }

            ps.close();

            return childPathList;

        } catch (Exception e) {

            String msg = "Failed to get child paths of resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    /**
     * Return the author (the user who creates the resource) if a resource exists at given path.
     * If a resource does not exist, null is returned.
     *
     * @param path Path of the resource.
     * @return Author if the resourc exists. Null otherwiese.
     * @throws RegistryException On any database related exception.
     */
    public String getResourceAuthor(String path) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "SELECT CREATOR FROM REG_RESOURCE WHERE PATH=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, path);

            ResultSet result = ps.executeQuery();

            String author = null;
            if (result.next()) {
                author = result.getString("CREATOR");
            }

            ps.close();

            return author;

        } catch (SQLException e) {

            String msg = "Failed to get the author of the resource " + path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void fillChildren(CollectionImpl collection, int start, int pageLen)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        // TODO - Cache the results in a more manageable way when using pagination!

        try {
            String sql = "SELECT R.RID, R.PATH FROM REG_RESOURCE R, REG_DEPENDENCY D WHERE D.PARENT_RID=?" +
                    " AND D.CHILD_RID=R.RID ORDER BY R.PATH";

            //String sql = "SELECT R.RID, R.PATH FROM REG_RESOURCE R " +
            //        "WHERE R.RID=(SELECT D.CHILD_RID FROM REG_DEPENDENCY D WHERE D.PARENT_RID=?) " +
            //        "ORDER BY R.PATH";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, collection.getId());

            List <String> childPathList = new ArrayList();
            ResultSet results = ps.executeQuery();
            int current = 0;
            int end = start + pageLen;
            while (results.next() && (pageLen == -1 || current <= end)) {
                String childID = results.getString("RID");
                if (AuthorizationUtils.authorize(childID, ActionConstants.GET)) {

                    if (current >= start && (pageLen == -1 || current < end)) {
                        childPathList.add(results.getString("PATH"));
                    }

                    current++;
                }
            }
            if (current < start) {
                throw new RegistryException("Didn't have enough results to start at #" + start);
            }
            ps.close();

            String[] childPaths = childPathList.toArray(new String[childPathList.size()]);
            collection.setContent(childPaths);

        } catch (SQLException e) {

            String msg = "Failed to get child paths of resource " +
                    collection.getPath() + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private ResourceImpl getResource(String path) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "SELECT RID, PATH, MEDIA_TYPE, COLLECTION, CREATOR, CREATED_TIME, " +
                    "LAST_UPDATOR, LAST_UPDATED_TIME, DESCRIPTION, CONTENT_ID, " +
                    "ASSOCIATED_SNAPSHOT_ID " +
                    "FROM REG_RESOURCE WHERE PATH=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, path);

            ResultSet result = ps.executeQuery();

            ResourceImpl resourceImpl = null;

            if (result.next()) {

                if (result.getInt("COLLECTION") == 0) {
                    resourceImpl = new ResourceImpl();
                } else {
                    resourceImpl = new CollectionImpl();
                }

                // this is always the current version of the resource
                resourceImpl.setVersionNumber(-1);

                resourceImpl.setId(result.getString("RID"));
                resourceImpl.setPath(result.getString("PATH"));
                resourceImpl.setMediaType(result.getString("MEDIA_TYPE"));
                resourceImpl.setAuthorUserName(result.getString("CREATOR"));
                resourceImpl.setCreatedTime(result.getTimestamp("CREATED_TIME"));
                resourceImpl.setLastUpdaterUserName(result.getString("LAST_UPDATOR"));
                resourceImpl.setLastModified(result.getTimestamp("LAST_UPDATED_TIME"));
                resourceImpl.setDescription(result.getString("DESCRIPTION"));
                resourceImpl.setDbBasedContentID(result.getString("CONTENT_ID"));
                resourceImpl.setMatchingSnapshotID(result.getLong("ASSOCIATED_SNAPSHOT_ID"));
            }

            ps.close();

            return resourceImpl;

        } catch (SQLException e) {

            String msg = "Failed to get the resource at path " + path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public ResourceImpl getResourceByID(String resourceID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "SELECT RID, PATH, MEDIA_TYPE, COLLECTION, CREATOR, CREATED_TIME, " +
                    "LAST_UPDATOR, LAST_UPDATED_TIME, DESCRIPTION, CONTENT_ID, " +
                    "ASSOCIATED_SNAPSHOT_ID " +
                    "FROM REG_RESOURCE WHERE RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResultSet result = ps.executeQuery();

            ResourceImpl resourceImpl = null;

            if (result.next()) {

                if (result.getInt("COLLECTION") == 0) {
                    resourceImpl = new ResourceImpl();
                } else {
                    resourceImpl = new CollectionImpl();
                }

                // this is always the current version of the resource
                resourceImpl.setVersionNumber(-1);

                resourceImpl.setId(result.getString("RID"));
                resourceImpl.setPath(result.getString("PATH"));
                resourceImpl.setMediaType(result.getString("MEDIA_TYPE"));
                resourceImpl.setAuthorUserName(result.getString("CREATOR"));
                resourceImpl.setCreatedTime(result.getTimestamp("CREATED_TIME"));
                resourceImpl.setLastUpdaterUserName(result.getString("LAST_UPDATOR"));
                resourceImpl.setLastModified(result.getTimestamp("LAST_UPDATED_TIME"));
                resourceImpl.setDescription(result.getString("DESCRIPTION"));
                resourceImpl.setFileBasedContentID(result.getString("CONTENT_ID"));
                resourceImpl.setMatchingSnapshotID(result.getLong("ASSOCIATED_SNAPSHOT_ID"));
            }

            ps.close();

            return resourceImpl;

        } catch (SQLException e) {

            String msg = "Failed to get the resource " + resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void createFileBasedContentFromDB(String contentID, Connection conn)
            throws RegistryException {

        if (FileManager.getInstance().lockFileBasedContent(contentID)) {
            // file based content is already created for this content ID. Locked it so that it will
            // not be automatically deleted.
            return;
        }

        try {
            String sql = "SELECT CONTENT_DATA FROM REG_CONTENT WHERE CONTENT_ID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, contentID);

            ResultSet result = ps.executeQuery();

            if (result.next()) {
                FileManager.getInstance().
                        createFileBasedContent(contentID, result.getBinaryStream("CONTENT_DATA"));
            }
            ps.close();

        } catch (SQLException e) {
            String msg = "Faild to get resource content from the database. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        }
    }

    /**
     * Returns a input stream to fetch content of the resource. Input stream is disconnected from
     * the database, so that it is possible to close the connection before reading the content
     * stream.
     *
     * This method is used to read content from within the resource implementation.
     * Note that this method does not get the connection from the current transaction. Therefore,
     * it is possible to use this method outside a transaction. Usually getting a content
     * from a resource is done after getting the resource. Thus it does not belong to the
     * transactions involving the Registry API.
     *
     * @param contentID UUID of the content
     * @param conn Database connection to the Registry database.
     * @return InputStream of the resource content
     * @throws RegistryException on any exception
     */
    public InputStream getResourceContentStream(String contentID, Connection conn)
            throws RegistryException {

        InputStream contentStream = FileManager.getInstance().getFileBasedInputStream(contentID);
        if (contentStream != null) {
            return contentStream;
        }

        try {
            String sql = "SELECT CONTENT_DATA FROM REG_CONTENT WHERE CONTENT_ID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, contentID);

            ResultSet result = ps.executeQuery();

            if (result.next()) {
                contentStream = FileManager.getInstance().
                        createFileBasedInputStream(contentID, result.getBinaryStream("CONTENT_DATA"));
            }
            ps.close();

            return contentStream;

        } catch (SQLException e) {
            String msg = "Faild to get resource content from the database. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        }
    }


    private void addResource(String path, ResourceImpl resourceImpl)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "INSERT INTO REG_RESOURCE (RID, PATH, MEDIA_TYPE, COLLECTION, CREATOR, " +
                    "CREATED_TIME, LAST_UPDATOR, LAST_UPDATED_TIME, DESCRIPTION, " +
                    "CONTENT_ID, EQUIVALENT_VERSION, ASSOCIATED_SNAPSHOT_ID) " +
                    "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

            long now = System.currentTimeMillis();

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceImpl.getId());
            ps.setString(2, path);
            ps.setString(3, resourceImpl.getMediaType());
            ps.setInt(4, resourceImpl instanceof Collection ? 1 : 0);
            ps.setString(5, CurrentSession.getUser());
            ps.setTimestamp(6, new Timestamp(now));
            ps.setString(7, CurrentSession.getUser());
            ps.setTimestamp(8, new Timestamp(now));
            ps.setString(9, resourceImpl.getDescription());
            ps.setString(10, resourceImpl.getFileBasedContentID() != null?
                    resourceImpl.getFileBasedContentID(): resourceImpl.getDbBasedContentID());
            ps.setLong(11, -1); // this is a new resource. so there is no equivalent version.

            // we are not aware of any snapshots created for this state of the resource. if a
            // snapshot is created, version repository is responsible for updating this feild.
            ps.setLong(12, -1);

            ps.executeUpdate();

            ps.close();

            // Update state of the ResourceImpl
            resourceImpl.setLastModified(new Date(now));
            resourceImpl.setPath(path);

        } catch (SQLException e) {

            String msg = "Failed to add resource to path " + path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private void updateResource(String resourceID, ResourceImpl resourceImpl)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "UPDATE REG_RESOURCE SET MEDIA_TYPE=?, LAST_UPDATOR=?, " +
                    "LAST_UPDATED_TIME=?, DESCRIPTION=?, CONTENT_ID=? WHERE RID=?";

            long now = System.currentTimeMillis();

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceImpl.getMediaType());
            ps.setString(2, CurrentSession.getUser());
            ps.setTimestamp(3, new Timestamp(now));
            ps.setString(4, resourceImpl.getDescription());
            ps.setString(5, resourceImpl.getDbBasedContentID() != null?
                resourceImpl.getDbBasedContentID(): resourceImpl.getFileBasedContentID());
            ps.setString(6, resourceID);

            ps.executeUpdate();

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to update the resource at path " +
                    resourceImpl.getPath() + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private void deleteResource(String resourceID) throws RegistryException {

        setParentEquivalentVersion(resourceID, -1);

        Connection conn = Transaction.getConnection();

        try {
            String sql = "DELETE FROM REG_RESOURCE WHERE RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ps.executeUpdate();

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to delete the resource " + resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private void setParentEquivalentVersion(String childID, int equivalentVersion)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "UPDATE REG_RESOURCE SET EQUIVALENT_VERSION=? " +
                    "WHERE RID=(SELECT D.PARENT_RID FROM REG_DEPENDENCY D WHERE D.CHILD_RID=?)";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setInt(1, equivalentVersion);
            ps.setString(2, childID);

            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to set the equivalent version of the parent of child resource " +
                    childID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private void addDependency(String parentID, String childID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "INSERT INTO REG_DEPENDENCY (PARENT_RID, CHILD_RID) VALUES (?, ?)";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, parentID);
            ps.setString(2, childID);

            ps.executeUpdate();

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to add the dependency between resource IDs " +
                    parentID + " and " + childID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private void addProperties(Resource resource) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "INSERT INTO REG_PROPERTY (RID, NAME, PROPERTY_VALUE) VALUES (?, ?, ?)";

            PreparedStatement ps = conn.prepareStatement(sql);

            String resourceID = resource.getId();
            Properties props = resource.getProperties();
            if (props != null) {

                Iterator i = props.keySet().iterator();
                while (i.hasNext()) {
                    String name = (String)i.next();

                    List propValues = (List) props.get(name);
                    if (propValues != null) {
                        Iterator<String> iValues = propValues.iterator();
                        while (iValues.hasNext()) {
                            String value = iValues.next();

                            ps.setString(1, resourceID);
                            ps.setString(2, name);
                            ps.setString(3, value);

                            ps.executeUpdate();
                            ps.clearParameters();
                        }
                    }
                }
            }

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to add properties to the resource " +
                    resource.getPath() + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private void removeProperties(ResourceImpl resourceImpl) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {

            String sql = "DELETE FROM REG_PROPERTY WHERE RID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceImpl.getId());

            ps.executeUpdate();

            ps.close();

        } catch (SQLException e) {

            String msg = "Failed tp remove properties of the resource " +
                    resourceImpl.getPath() + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private void updateProperties(ResourceImpl resourceImpl) throws RegistryException {

        removeProperties(resourceImpl);
        addProperties(resourceImpl);
    }

    /**
     * Adds the content of the given input stream to the REG_CONTENT tabel and returns a UUID to
     * identify that content. This content is independent and can be refered by any resource.
     *
     * @param resourceImpl Resource of which the content has to be written
     * @throws RegistryException if there is any exception while adding the content.
     */
    private void addContent(ResourceImpl resourceImpl) throws RegistryException {

        String currentContentID = resourceImpl.getDbBasedContentID() != null?
                resourceImpl.getDbBasedContentID(): resourceImpl.getFileBasedContentID();
        if (contentExists(currentContentID)) {
            // content with the current content of the resource already exists in the database.
            // since content IDs are unique, both contents are same and we don't have to add
            // the content again.
            return;
        }

        Connection conn = Transaction.getConnection();

        InputStream contentStream = resourceImpl.getContentStream();
        if (contentStream != null) {

            try {
                String contentID = currentContentID;
                long contentLength = FileManager.getInstance().getContentLength(contentID);

                String sql = "INSERT INTO REG_CONTENT (CONTENT_ID, CONTENT_DATA) VALUES (?, ?)";

                PreparedStatement ps = conn.prepareStatement(sql);
                ps.setString(1, contentID);
                ps.setBinaryStream(2, contentStream, (int)contentLength);
                ps.executeUpdate();
                ps.close();

            } catch (SQLException e) {

                String msg = "Failed to write resource content to the database. " + e.getMessage();
                log.error(msg, e);
                throw new RegistryException(msg, e);
            } finally {
                if (contentStream != null) {
                    try {
                        contentStream.close();
                    } catch (IOException e) {
                        log.error(e);
                    }
                }
            }
        }
    }

    private boolean contentExists(String contentID) throws RegistryException {

        String sql = "SELECT CONTENT_ID FROM REG_CONTENT WHERE CONTENT_ID=?";

        Connection conn = Transaction.getConnection();

        boolean contentExists = false;
        try {
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, contentID);

            ResultSet result = ps.executeQuery();
            if (result.next()) {
                contentExists = true;
            }
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to check existance of content with ID " +
                    contentID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return contentExists;
    }

    public void updateContent(String contentID, InputStream contentStream)
            throws RegistryException {

        FileManager.getInstance().deleteFileBasedContent(contentID);

        Connection conn = Transaction.getConnection();

        try {

            InputStream tempFileStream = FileManager.getInstance().
                    createFileBasedInputStream(contentID, contentStream);
            long contentLength = FileManager.getInstance().getContentLength(contentID);

            String sql = "UPDATE REG_CONTENT SET CONTENT_DATA=? WHERE CONTENT_ID=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setBinaryStream(1, tempFileStream, (int)contentLength);
            ps.setString(2, contentID);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to write resource content to the database. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    /**
     * Content of resources are stored seprately in the REG_CONTENT table and given a UUID. Resource
     * refer to thier contents by this UUID. Multiple resources may refer to a same content.
     * Therefore, when deleting a resource, we can't simply delete its content. We have check if
     * the content of the deleted resource is referred by other resources. If it is referred by
     * other resources, content should NOT be deleted. If the content is not referred by any other
     * resource, we are free to delete that content.
     *
     * @param contentID ID of the content
     * @throws RegistryException On database related exceptions.
     */
    private void deleteContent(String contentID) throws RegistryException {

        if (getContentUsage(contentID) > 0) {
            return;
        }

        Connection conn = Transaction.getConnection();

        try {
            String sql = "DELETE FROM REG_CONTENT WHERE CONTENT_ID=?";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, contentID);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to delete content with the ID " +
                    contentID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    private long getContentUsage(String contentID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        String sql = "SELECT COUNT(CONTENT_ID) AS USAGE FROM REG_RESOURCE WHERE CONTENT_ID=?";

        try {
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, contentID);

            long usage = 0;

            ResultSet result = ps.executeQuery();
            if (result.next()) {
                usage = result.getLong("USAGE");
            }
            ps.close();

            return usage;

        } catch (SQLException e) {

            String msg = "Failed to get content usage for the content ID " +
                    contentID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void setPath(String resourceID, String newPath) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "UPDATE REG_RESOURCE SET PATH=? WHERE RID=?";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, newPath);
            ps.setString(2, resourceID);

            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to change the path of the resource " +
                    newPath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public long getEquivalenVersion(String resourceID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "SELECT EQUIVALENT_VERSION FROM REG_RESOURCE WHERE RID=?";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResultSet result = ps.executeQuery();
            long equivalentVersion = -1;
            if (result.next()) {
                equivalentVersion = result.getLong("EQUIVALENT_VERSION");
            }
            ps.close();

            return equivalentVersion;

        } catch (SQLException e) {

            String msg = "Failed to get equivalent version of the resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public ResourceDO getResourceDO(String resourceID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        String sql = "SELECT R.RID, R.EQUIVALENT_VERSION, R.PATH, R.MEDIA_TYPE, R.COLLECTION, " +
                "R.CREATOR, R.CREATED_TIME, R.LAST_UPDATOR, R.LAST_UPDATED_TIME, " +
                "R.DESCRIPTION, R.CONTENT_ID, R.ASSOCIATED_SNAPSHOT_ID " +
                "FROM REG_RESOURCE R WHERE R.RID=?";

        try {
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);

            ResourceDO resourceDO = null;
            ResultSet result = ps.executeQuery();
            if (result.next()) {
                resourceDO = new ResourceDO();

                resourceDO.setID(result.getString("RID"));
                resourceDO.setEquivalentVersion(result.getLong("EQUIVALENT_VERSION"));
                resourceDO.setPath(result.getString("PATH"));
                resourceDO.setMediaType(result.getString("MEDIA_TYPE"));
                resourceDO.setCollection(result.getInt("COLLECTION"));
                resourceDO.setAuthor(result.getString("CREATOR"));
                resourceDO.setCreatedOn(result.getTimestamp("CREATED_TIME"));
                resourceDO.setLastUpdator(result.getString("LAST_UPDATOR"));
                resourceDO.setLastUpdatedOn(result.getTimestamp("LAST_UPDATED_TIME"));
                resourceDO.setDescription(result.getString("DESCRIPTION"));
                resourceDO.setContentID(result.getString("CONTENT_ID"));
                resourceDO.setAssociatedSnapshotID(result.getLong("ASSOCIATED_SNAPSHOT_ID"));
            }

            return resourceDO;

        } catch (SQLException e) {

            String msg = "Failed to read resource version data for resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public String addContent(InputStream contentStream) throws RegistryException {

        String contentID = UUIDGenerator.generateUUID();

        Connection conn = Transaction.getConnection();

        try {

            InputStream tempFileStream =
                    FileManager.getInstance().createFileBasedInputStream(contentID, contentStream);
            long contentLength = FileManager.getInstance().getContentLength(contentID);

            String sql = "INSERT INTO REG_CONTENT (CONTENT_ID, CONTENT_DATA) VALUES (?, ?)";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, contentID);
            ps.setBinaryStream(2, tempFileStream, (int)contentLength);
            ps.executeUpdate();
            ps.close();

            return contentID;

        } catch (SQLException e) {

            String msg = "Failed to write resource content to the database. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void setSnapshotID(String resourceID, long snapshotID) throws RegistryException {

        Connection conn = Transaction.getConnection();

        String sql = "UPDATE REG_RESOURCE SET ASSOCIATED_SNAPSHOT_ID=? WHERE RID=?";

        try {
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setLong(1, snapshotID);
            ps.setString(2, resourceID);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to set the snapshot ID for the resource " +
                    resourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

    }

    public void markDirtyByPath(String resourcePath) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "UPDATE REG_RESOURCE SET EQUIVALENT_VERSION=?, ASSOCIATED_SNAPSHOT_ID=? " +
                    "WHERE PATH=?";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setLong(1, -1);
            ps.setLong(2, -1);
            ps.setString(3, resourcePath);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to update the equivalent version of the resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void invalidateCurrentSnapshot(String resourcePath) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "UPDATE REG_RESOURCE SET ASSOCIATED_SNAPSHOT_ID=? WHERE PATH=?";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setLong(1, -1);
            ps.setString(2, resourcePath);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to invalidate the current snapshot of the resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }
}
