/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.Tag;
import org.wso2.registry.TaggedResourcePath;
import org.wso2.registry.jdbc.DatabaseConstants;
import org.wso2.registry.jdbc.dataobjects.TaggingDO;
import org.wso2.registry.jdbc.utils.Transaction;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;

public class TagsDAO {

    private static final Log log = LogFactory.getLog(TagsDAO.class);

    public static void addTagging(String tagName, String resourceID, String userID)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "INSERT INTO REG_TAG (TAG_NAME, RID, USER_ID, TAGGED_TIME) VALUES (?,?,?,?)";

            long now = System.currentTimeMillis();

            PreparedStatement s = conn.prepareStatement(sql);
            s.setString(1, tagName);
            s.setString(2, resourceID);
            s.setString(3, userID);
            s.setDate(4, new Date(now));

            s.executeUpdate();
            s.close();

        } catch (SQLException e) {

            String msg = "Failed to add tag " + tagName + " to resource " + resourceID +
                    " by user " + userID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public static void copyTags(String fromResourceID , String toResourceID)
            throws RegistryException{

        Connection conn = Transaction.getConnection();

        ArrayList tagList = new ArrayList();
        try {
            String SQL = "SELECT TAG_NAME , USER_ID , TAGGED_TIME FROM REG_TAG WHERE RID=?";
            PreparedStatement s = conn.prepareStatement(SQL);
            s.setString(1, fromResourceID);
            ResultSet result = s.executeQuery();
            while (result.next()) {
                TaggingDO  taggingDO = new TaggingDO();
                taggingDO.setTagName(result.getString(1));
                taggingDO.setTaggedUserName(result.getString(2));
                taggingDO.setTaggedTime(result.getDate(3));
                tagList.add(taggingDO);
            }

            s.close();

        } catch (SQLException e) {
            String msg = "Failed to get tags of resource " + fromResourceID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        for (Object aTagList : tagList) {
            TaggingDO taggingDO = (TaggingDO)aTagList;
            addTagging(taggingDO.getTagName(), toResourceID, taggingDO.getTaggedUserName());
        }
    }

    public static boolean taggingExists(String tagName, String resourceID, String userID)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        String sql =
                "SELECT TAG_ID FROM REG_TAG WHERE TAG_NAME=? AND RID=? AND USER_ID=?";

        try {
            PreparedStatement s = conn.prepareStatement(sql);
            s.setString(1, tagName);
            s.setString(2, resourceID);
            s.setString(3, userID);

            ResultSet result = s.executeQuery();

            boolean tagExists = false;
            if (result.next()) {
                tagExists = true;
            }

            s.close();

            return tagExists;

        } catch (SQLException e) {

            String msg = "Failed to check the existence of the tag " + tagName +
                    " on resource " + resourceID + " by user " + userID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

    }

    public static void removeTag(String path, String tag)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        ResourceDAO resourceDAO = new ResourceDAO();
        String resourceID = resourceDAO.getResourceID(path);

        try {
            String sql = "DELETE FROM REG_TAG WHERE RID=? AND TAG_NAME=?";

            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setString(1, resourceID);
            ps.setString(2, tag);

            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to remove tag " + tag + " from the resource " +
                    path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public static void removeTag(String path, String tag, String userName)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        ResourceDAO resourceDAO = new ResourceDAO();
        String resourceID = resourceDAO.getResourceID(path);

        try {

            String sql = "DELETE FROM REG_TAG WHERE RID=? AND TAG_NAME=?";
            if (!userName.equals("*")) {
                sql = sql + "AND USER_ID=?";
            }

            PreparedStatement s = conn.prepareStatement(sql);
            s.setString(1, resourceID);
            s.setString(2, tag);
            if (!userName.equals("*")) {
                s.setString(3, userName);
            }

            s.executeUpdate();
            s.close();

        } catch (SQLException e) {

            String msg = "Failed to remove tag " + tag + " on resource " + path +
                    " done by user " + userName + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public static String[] getPathsWithTag(String tag, Connection conn) throws SQLException {

        String sql = "SELECT A.PATH FROM ARTIFACTS A, TAGS T WHERE T.TAG_NAME=? AND T.AID=A.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        ResultSet result = s.executeQuery();
        List pathList = new ArrayList();
        while (result.next()) {
            pathList.add(result.getString(DatabaseConstants.PATH_FIELD));
        }

        return (String[])pathList.toArray(new String[pathList.size()]);
    }

    public static String[] getTags(String path) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "SELECT T.TAG_NAME FROM REG_TAG T, REG_RESOURCE R WHERE R.PATH=? AND R.RID=T.RID";

            PreparedStatement s = conn.prepareStatement(sql);
            s.setString(1, path);

            ResultSet result = s.executeQuery();
            List tagList = new ArrayList();
            while (result.next()) {
                tagList.add(result.getString(DatabaseConstants.TAG_NAME_FIELD));
            }

            return (String[])tagList.toArray(new String[tagList.size()]);

        } catch (SQLException e) {

            String msg = "Failed to get tags associated with the resource path " +
                    path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public static List getPathsWithAnyTag(String[] tags) throws RegistryException {

        Connection conn = Transaction.getConnection();

        String sql;
        if (tags.length == 1) {
            sql = "SELECT DISTINCT R.PATH FROM REG_RESOURCE R, REG_TAG T " +
                    "WHERE R.RID=T.RID AND T.TAG_NAME=?";
        } else if (tags.length == 2) {
            sql = "SELECT DISTINCT R.PATH FROM REG_RESOURCE R, REG_TAG T " +
                    "WHERE R.RID=T.RID AND (T.TAG_NAME=? OR T.TAG_NAME=?)";
        } else if (tags.length == 3) {
            sql = "SELECT DISTINCT R.PATH FROM REG_RESOURCE R, REG_TAG T " +
                    "WHERE R.RID=T.RID AND (T.TAG_NAME=? OR T.TAG_NAME=? OR T.TAG_NAME=?)";
        } else {
            String msg = "Tag queries with more than three tags are not supported.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        List resourcePaths = new ArrayList();
        try {
            PreparedStatement s = conn.prepareStatement(sql);

            for (int i = 0; i < tags.length; i++) {
                s.setString(i + 1, tags[i]);
            }

            ResultSet results = s.executeQuery();
            while (results.next()) {
                resourcePaths.add(results.getString(DatabaseConstants.PATH_FIELD));
            }
            s.close();

        } catch (SQLException e) {

            String msg = "Failed to resource paths with any of the tags " +
                         Arrays.toString(tags) + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return resourcePaths;
    }

    public static long getTagCount(String path, String tag) throws RegistryException {

        Connection conn = Transaction.getConnection();

        String sql =
                "SELECT COUNT(T.TAG_NAME) FROM REG_RESOURCE R, REG_TAG T " +
                        "WHERE R.PATH=? AND T.TAG_NAME=? AND T.RID=R.RID GROUP BY R.PATH";

        try {
            PreparedStatement s = conn.prepareStatement(sql);
            s.setString(1, path);
            s.setString(2, tag);

            ResultSet result = s.executeQuery();

            long tagCount = 0;
            if (result.next()) {
                tagCount = result.getLong(1);
            }

            s.close();

            return tagCount;

        } catch (SQLException e) {

            String msg = "Failed to get tag count of tag " + tag +
                    " on resource " + path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
         }
    }

    public static List getTaggedPaths(String tag, Connection conn) throws SQLException {

        String sql =
                "SELECT A.PATH, COUNT(TN.TAG_NAME) FROM ARTIFACTS A, TAGS TN WHERE TN.TAG_NAME=? AND TN.AID=A.AID GROUP BY A.PATH";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        ResultSet result = s.executeQuery();
        List pathList = new ArrayList();
        while (result.next()) {
            TaggedResourcePath taggedResourcePath = new TaggedResourcePath();
            taggedResourcePath.setResourcePath(result.getString(DatabaseConstants.PATH_FIELD));
            taggedResourcePath.setTagCount(result.getLong(2));
            pathList.add(taggedResourcePath);
        }

        return pathList;
    }

    public static Tag[] getTagsWithCount(String path) throws RegistryException {

        Connection conn = Transaction.getConnection();

        List tagList = new ArrayList();

        try {
            String sql = "SELECT T.TAG_NAME, COUNT(R.RID) FROM REG_TAG T, REG_RESOURCE R " +
                    "WHERE R.PATH=? AND R.RID=T.RID GROUP BY T.TAG_NAME";

            PreparedStatement s = conn.prepareStatement(sql);
            s.setString(1, path);

            ResultSet result = s.executeQuery();
            while (result.next()) {
                Tag tag = new Tag();
                tag.setTagName(result.getString(DatabaseConstants.TAG_NAME_FIELD));
                tag.setTagCount(result.getLong(2));
                tagList.add(tag);
            }
            s.close();

        } catch (SQLException e) {

            String msg = "Failed to get tags and tag counts of the resource " +
                    path + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return (Tag[])tagList.toArray(new Tag[tagList.size()]);
    }

    public static TaggingDO getTagging(String path, String tagName, String userName)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        ResourceDAO resourceDAO = new ResourceDAO();
        String resourceID = resourceDAO.getResourceID(path);

        try {
            TaggingDO taggingDO = null;

            String sql = "SELECT T.TAGGED_TIME FROM REG_TAG T, REG_RESOURCE R " +
                "WHERE R.RID=T.RID AND T.RID=? AND T.USER_ID=? AND T.TAG_NAME=?";

            PreparedStatement s = conn.prepareStatement(sql);
            s.setString(1, resourceID);
            s.setString(2, userName);
            s.setString(3, tagName);

            ResultSet result = s.executeQuery();
            if (result.next()) {

                java.util.Date taggedTime = new java.util.Date(
                        result.getTimestamp(DatabaseConstants.TAGGED_TIME_FIELD).getTime());

                taggingDO = new TaggingDO();
                taggingDO.setResourceID(resourceID);
                taggingDO.setResourcePath(path);
                taggingDO.setTagName(tagName);
                taggingDO.setTaggedTime(taggedTime);
                taggingDO.setTaggedUserName(userName);
            }
            s.close();

            return taggingDO;

        } catch (SQLException e) {

            String msg = "Failed to get tagging information for tag " + tagName +
                    " on resource " + path + " by user " + userName + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public static TaggingDO getTagging(long taggingID)
            throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String sql = "SELECT R.RID, R.PATH, T.USER_ID, T.TAG_NAME, T.TAGGED_TIME " +
                    "FROM REG_TAG T, REG_RESOURCE R WHERE R.RID=T.RID AND T.TAG_ID=?";

            PreparedStatement s = conn.prepareStatement(sql);
            s.setLong(1, taggingID);

            TaggingDO taggingDO = null;
            ResultSet result = s.executeQuery();
            if (result.next()) {

                java.util.Date taggedTime = new java.util.Date(
                        result.getTimestamp(DatabaseConstants.TAGGED_TIME_FIELD).getTime());

                taggingDO = new TaggingDO();
                taggingDO.setResourceID(result.getString(DatabaseConstants.AID_FIELD));
                taggingDO.setResourcePath(result.getString(DatabaseConstants.PATH_FIELD));
                taggingDO.setTagName(result.getString(DatabaseConstants.TAG_NAME_FIELD));
                taggingDO.setTaggedUserName(result.getString(DatabaseConstants.USER_ID_FIELD));
                taggingDO.setTaggedTime(taggedTime);
            }
            s.close();

            return taggingDO;

        } catch (SQLException e) {

            String msg = "Failed to get tagging information for tag ID " +
                    taggingID + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }
}
