/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.filecache;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.exceptions.RegistryException;

import java.io.*;
import java.util.*;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.concurrent.locks.Lock;

public class FileManager {

    private static Log log = LogFactory.getLog(FileManager.class);

    private static final long MAXIMUM_IDLE_TIME_FOR_FILE = 30 * 60 * 1000; // 30 minutes

    private static final long PREFERRED_FILE_COUNT = 100;

    private final ReadWriteLock lock = new ReentrantReadWriteLock(true);
    private final Lock r = lock.readLock();
    private final Lock w = lock.writeLock();

    private Map <String, FileData> fileDataMap =
            Collections.synchronizedMap(new HashMap <String, FileData> ());

    private static FileManager fileManager = new FileManager();

    public static FileManager getInstance() {
        return fileManager;
    }

    private FileManager() {}

    public void init() {

        long oneHour = 60 * 60 * 1000;
        Timer timer = new Timer(true);
        timer.scheduleAtFixedRate(new FileCleanerTask(), oneHour, oneHour);
    }

    public InputStream getFileBasedInputStream(String contentID) throws RegistryException {

        r.lock();

        try {

            FileData fileData = fileDataMap.get(contentID);
            File file;
            if (fileData != null) {
                file = new File(fileData.getFilename());
                if (!file.exists()) {
                    //fileDataMap.remove(contentID);
                    return null;
                }
            } else {
                return null;
            }

            fileData.incrementConnection();

            InputStream inputStream;
            try {
                inputStream = new RegistryFileInputStream(
                        new BufferedInputStream(new FileInputStream(file)), contentID);

                return inputStream;
            } catch (FileNotFoundException e) {
                String msg = "Failed to create an input stream from the temp file " +
                        fileData.getFilename() + ". " + e.getMessage();
                log.error(msg, e);

                throw new RegistryException(msg, e);
            }

        } finally {
            r.unlock();
        }
    }

    public boolean lockFileBasedContent(String contentID) {

        FileData fileData = fileDataMap.get(contentID);
        if (fileData != null) {
            fileData.incrementActiveResource();
            return true;
        }

        return false;
    }

    public void createFileBasedContent(String contentID, InputStream inputStream)
            throws RegistryException {

        w.lock();

        try {

            if (inputStream == null) {
                String msg = "Could not create file based content for null input stream.";
                log.error(msg);
                throw new RegistryException(msg);
            }

            FileData fileData = fileDataMap.get(contentID);
            if (fileData != null) {
                File file = new File(fileData.getFilename());
                if (!file.exists())
                    fileData = null;
            }

            if (fileData == null) {
                BufferedOutputStream out = null;
                try {
                    File tempFile = File.createTempFile("reg", ".bin");
                    tempFile.deleteOnExit();

                    out = new BufferedOutputStream(new FileOutputStream(tempFile));
                    byte[] contentChunk = new byte[1024];
                    int byteCount;
                    while ((byteCount = inputStream.read(contentChunk)) != -1) {
                        out.write(contentChunk, 0, byteCount);
                    }
                    out.flush();

                    // we increment the file data only after obtaining the input stream from file
                    fileData = new FileData(tempFile.getPath());
                    fileData.incrementActiveResource();
                    fileDataMap.put(contentID, fileData);

                } catch (IOException e) {
                    String msg = "Failed to write data to temporary file." + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);
                } finally {
                    try {
                        inputStream.close();
                        if (out != null) {
                            out.close();
                        }
                    } catch (IOException e) {
                        String msg = "Failed to close streams used for temporary file. "
                                + e.getMessage();
                        log.error(msg, e);
                    }
                }
            }

        } finally {
            w.unlock();
        }
    }

    public InputStream createFileBasedInputStream(String contentID,
                                                  InputStream inputStream)
            throws RegistryException {

        w.lock();

        try {

            if (inputStream == null) {
                return null;
            }

            FileData fileData = fileDataMap.get(contentID);
            if (fileData != null) {
                File file = new File(fileData.getFilename());
                if (!file.exists())
                    fileData = null;
            }

            if (fileData == null) {
                BufferedOutputStream out = null;
                try {
                    File tempFile = File.createTempFile("reg", ".bin");
                    tempFile.deleteOnExit();

                    out = new BufferedOutputStream(new FileOutputStream(tempFile));
                    byte[] contentChunk = new byte[1024];
                    int byteCount;
                    while ((byteCount = inputStream.read(contentChunk)) != -1) {
                        out.write(contentChunk, 0, byteCount);
                    }
                    out.flush();

                    // we increment the file data only after obtaining the input stream from file
                    fileData = new FileData(tempFile.getPath());
                    fileDataMap.put(contentID, fileData);

                } catch (IOException e) {
                    String msg = "Failed to write data to temporary file." + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);
                } finally {
                    try {
                        inputStream.close();
                        if (out != null) {
                            out.close();
                        }
                    } catch (IOException e) {
                        String msg = "Failed to close streams used for temporary file. "
                                + e.getMessage();
                        log.error(msg, e);
                    }
                }
            }

            try {
                InputStream fileBasedInputStream =
                        new RegistryFileInputStream(new BufferedInputStream(
                                new FileInputStream(fileData.getFilename())), contentID);
                fileData.incrementConnection();
                return fileBasedInputStream;
            } catch (FileNotFoundException e) {
                String msg = "Failed to create an input stream from the temp file " +
                        fileData.getFilename() + ". " + e.getMessage();
                log.error(msg, e);
                throw new RegistryException(msg, e);
            }

        } finally {
            w.unlock();
        }
    }

    public long getContentLength(String contentID) {

        r.lock();

        try {
            FileData fileData = fileDataMap.get(contentID);
            if (fileData == null)
                return 0;
            File file = new File(fileData.getFilename());
            return file.exists() ? file.length() : 0;

        } finally {
            r.unlock();
        }
    }

    public void onStreamClosed(String contentID) {

        w.lock();

        try {

            FileData fileData = fileDataMap.get(contentID);
            if (fileData != null) {
                fileData.decrementConnection();
                if (fileData.getConnections() <= 0 && fileDataMap.size() > PREFERRED_FILE_COUNT) {
                    deleteFileBasedContent(contentID);
                }
            }

        } finally {
            w.unlock();
        }
    }

    public void cleanupFiles() {

        w.lock();

        try {
            long currentTime = System.currentTimeMillis();

            List <String> expiredContentList = new ArrayList <String> ();

            Iterator <String> contentIDs = fileDataMap.keySet().iterator();

            while (contentIDs.hasNext()) {

                String contentID = contentIDs.next();

                FileData fileData = fileDataMap.get(contentID);
                if (fileData.getActiveResourceCount() <= 0 && fileData.getConnections() <= 0 &&
                        (currentTime - fileData.getIdleFrom()) >= MAXIMUM_IDLE_TIME_FOR_FILE) {

                    // We can't call deleteFileBasedContent(...) here, since it will remove the
                    // contentID from the fileDataMap. This causes a ConcurrentModificationException
                    // as it tries to remove an element from a Map inside an iterator. To solve
                    // this, we have to populate the expired contentIDs to a separare list and
                    // remove them after the iteration is completed.
                    expiredContentList.add(contentID);
                }
            }

            Iterator <String> expired = expiredContentList.iterator();
            while (expired.hasNext()) {
                String expiredContentID = expired.next();
                deleteFileBasedContent(expiredContentID);
            }

        } finally {
            w.unlock();
        }
    }

    public void deleteFileBasedContent(String contentID) {

        w.lock();

        try {
            File file = getFile(contentID);

            if (file != null) {
                file.delete();
            }

            fileDataMap.remove(contentID);

        } finally {
            w.unlock();
        }
    }

    public void decrementActiveUsage(String contentID) {
        FileData fileData = fileDataMap.get(contentID);
        if (fileData != null) {
            fileData.decrementActiveResource();
        }
    }

    private File getFile(String contentID) {

        r.lock();

        try {
            FileData fileData = fileDataMap.get(contentID);
            if (fileData == null)
                return null;
            File file = new File(fileData.getFilename());
            return file.exists() ? file : null;

        } finally {
            r.unlock();
        }
    }
}
