/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers;

import org.wso2.registry.Collection;
import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.jdbc.Repository;

import java.util.Map;
import java.util.HashMap;

/**
 * Objects of this class contains the information about the current request to the registry.
 * All attributes of such objects should be either null or should contain a *valid" value. That is,
 * if an attribute contains a non-null value it is safe to assume that attribute contains a valid
 * value as mentioned by the javadoc of that attribute.
 */
public class RequestContext {

    /**
     * Specifies whether the processing of the current request is completed or not. If this is true,
     * any further processing of the request is not done.
     */
    private boolean processingComplete;

    /**
     * Path of the currently processing resource as given by the client of the Registry API
     */
    private ResourcePath resourcePath;

    /**
     * Path of the currently processing resource, which may have been modified by Registry impls to
     * represent the actual path in which resource resides.
     */
    private String actualPath;

    /**
     * Currently processing resource. This is the resource referred by the actual path.
     */
    private Resource resource;

    /**
     * URL where the current resource content is located.
     */
    private String sourceURL;

    /**
     * Path of the parent collection of the current resource.
     */
    private String parentPath;

    /**
     * Parent collection of the current resource.
     */
    private Collection parentCollection;

    private Map<String, Object> properties = new HashMap<String, Object>();

    private Registry registry;
    private Repository repository;

    public RequestContext(Registry registry, Repository repository) {
        this.registry = registry;
        this.repository = repository;
    }

    public Registry getRegistry() {
        return registry;
    }

    public Repository getRepository() {
        return repository;
    }

    public boolean isProcessingComplete() {
        return processingComplete;
    }

    public void setProcessingComplete(boolean processingComplete) {
        this.processingComplete = processingComplete;
    }

    public ResourcePath getResourcePath() {
        if (resourcePath == null && resource != null) {
            resourcePath = new ResourcePath(resource.getPath());
        }
        return resourcePath;
    }

    public void setResourcePath(ResourcePath resourcePath) {
        this.resourcePath = resourcePath;
    }

    public String getSourceURL() {
        return sourceURL;
    }

    public void setSourceURL(String sourceURL) {
        this.sourceURL = sourceURL;
    }

    public String getActualPath() {
        return actualPath;
    }

    public void setActualPath(String actualPath) {
        this.actualPath = actualPath;
    }

    public Resource getResource() {
        return resource;
    }

    public void setResource(Resource resource) {
        this.resource = resource;
    }

    public String getParentPath() {
        return parentPath;
    }

    public void setParentPath(String parentPath) {
        this.parentPath = parentPath;
    }

    public Collection getParentCollection() {
        return parentCollection;
    }

    public void setParentCollection(Collection parentCollection) {
        this.parentCollection = parentCollection;
    }

    public void setProperty(String name, Object value) {
        properties.put(name, value);
    }

    public Object getProperty(String name) {
        return properties.get(name);
    }
}
