/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers.builtin;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.dao.CommentsDAO;
import org.wso2.registry.jdbc.dao.ResourceDAO;
import org.wso2.registry.jdbc.handlers.Handler;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.session.CurrentSession;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.utils.AuthorizationUtils;

/**
 * Handles paths of the form <pure resource path>;comments:<comment ID>
 * e.g. /projects/ids/config.xml;comments:2
 */
public class CommentURLHandler extends Handler {

    private static final Log log = LogFactory.getLog(CommentURLHandler.class);

    public CommentURLHandler() {}

    public Resource get(RequestContext requestContext) throws RegistryException {

        ResourcePath resourcePath = requestContext.getResourcePath();

        String commentID = resourcePath.getParameterValue("comments");
        if (commentID != null) {

            long cID;
            try {
                cID = Long.parseLong(commentID);

            } catch (NumberFormatException e) {
                // note that this might NOT be an exceptional scenario. there could be a different URL
                // form, which contains strings after "comment".
                // it is just that it is not the URL we expect here
                return null;
            }

            Comment comment = CommentsDAO.getComment(cID);

            if (comment == null) {
                String msg = "Requested comment with ID: " + cID + " is not available.";
                log.info(msg);
                throw new RegistryException(msg);
            }

            requestContext.setProcessingComplete(true);
            return comment;

        }

        return null;
    }

    public void put(RequestContext requestContext) throws RegistryException {
    }

    public void importResource(RequestContext requestContext) throws RegistryException {
    }

    public void delete(RequestContext requestContext) throws RegistryException {

        requestContext.setProcessingComplete(false);
        ResourcePath resourcePath = requestContext.getResourcePath();

        String commentID = resourcePath.getParameterValue("comments");
        if (commentID != null) {

            long cID;
            try {
                cID = Long.parseLong(commentID);

            } catch (NumberFormatException e) {
                // note that this might not be an exceptional scenario. there could be a different URL
                // form, which contains strings after "comment".
                // it is just that it is not the URL we expect here
                return;
            }

            String userID = CurrentSession.getUser();
            String authPath = AuthorizationUtils.getAuthorizationPath(resourcePath.getPath());
            String commentAuthor;

            Comment comment = CommentsDAO.getComment(cID);
            commentAuthor = comment.getUser();

            String resourceID = ResourceDAO.getResourceID(authPath);

            // check if the current user has permission to delete this comment.
            // users who have PUT permission on the commented resource can delete any comment on
            // that resource. Any user can delete his own comment.

            try {
                UserRealm realm = CurrentSession.getRealm();

                if (!userID.equals(commentAuthor) &&
                        !realm.getAuthorizer().isUserAuthorized(userID, resourceID,
                                ActionConstants.PUT)) {

                    String msg = "User: " + userID +
                            " is not authorized to delete the comment on the resource: " +
                            authPath;
                    log.info(msg);
                    throw new AuthorizationFailedException(msg);
                }

            } catch (UserStoreException e) {
                //
            }

            CommentsDAO.deleteComment(cID);

            requestContext.setProcessingComplete(true);
        }
    }

    public void putChild(RequestContext requestContext) throws RegistryException {
    }

    public void importChild(RequestContext requestContext) throws RegistryException {

    }
}
