/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.queries;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.DatabaseConstants;
import org.wso2.registry.jdbc.dao.RatingsDAO;
import org.wso2.registry.jdbc.dao.TagsDAO;
import org.wso2.registry.jdbc.dataobjects.RatingDO;
import org.wso2.registry.jdbc.dataobjects.TaggingDO;
import org.wso2.registry.users.UserRealm;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class SQLQueryProcessor extends QueryProcessor {

    private static final Log log = LogFactory.getLog(SQLQueryProcessor.class);

    public SQLQueryProcessor(DataSource dataSource, UserRealm realm) {
        super(dataSource, realm);
    }

    public Collection executeQuery(Registry registry, Resource query, Map parameters) throws RegistryException {

        Connection conn = null;
        String keywords = null;
        String [] contentResult = null;

        try {

            String sqlString = (String)query.getContent();

            conn = dataSource.getConnection();
            PreparedStatement s = conn.prepareStatement(sqlString);

            if (parameters != null) {
                keywords = (String)parameters.get("content");
                parameters.remove("content");
                Iterator iParams = parameters.keySet().iterator();
                while (iParams.hasNext()) {
                    String paramNumber = (String)iParams.next();
                    Object paramValue = parameters.get(paramNumber);
                    s.setObject(Integer.parseInt(paramNumber), paramValue);
                }

                //for (int i = 0; i < parameters.length; i++) {
                //    s.setObject(i + 1, parameters[i]);
                //}
            }

            ResultSet results = s.executeQuery();
            if(keywords != null && !keywords.equals("")){
                contentResult = (String []) registry.searchContent(keywords).getContent();
            }

            String resultType = query.getProperty(RegistryConstants.RESULT_TYPE_PROPERTY_NAME);

            if (resultType == null || resultType.equals(RegistryConstants.RESOURCES_RESULT_TYPE)) {

                // Result is a normal resource, which is stored in the Resources table or a collection
                // of normal resources.

                return fillResourcesCollection(contentResult, results, conn);

            } else if (resultType.equals(RegistryConstants.COMMENTS_RESULT_TYPE)) {

                return fillCommentsCollection(results, conn);

            } else if (resultType.equals(RegistryConstants.RATINGS_RESULT_TYPE)) {

                return fillRatingsCollection(results, conn);

            } else if (resultType.equals(RegistryConstants.TAGS_RESULT_TYPE)) {

                return fillTagsCollection(results);

            } else if (resultType.equals("TagsWithCount")) {

                // Result is in the form of (tag, number of taggings) tuples
                // Fill TagCount objects with results and add them as an array


            } else if (resultType.equals("ResourcePathsWithTagCount")) {

                // Result is in the form of (resource path, number of taggings) tuples
                // Fill ResourceTagCount objects with results and add them as an array

            }

            String msg = "Unknown result type: " + resultType + " defined for the query: " +
                         sqlString + " located in path: " + query.getPath();
            log.error(msg);
            throw new RegistryException(msg);

        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());

        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignore) {
                    log.info("exception.closing.db");
                }
            }
        }
    }

    private Collection fillResourcesCollection(String [] contentResult, ResultSet results, Connection conn)
            throws SQLException {

        // Result is a normal resource, which is stored in the Resources table or a collection
        // of normal resources.

        List pathList = new ArrayList();
        if (contentResult != null){
        while (results.next()) {
            String path = results.getString(DatabaseConstants.PATH_FIELD);
            for(int i=0; i<contentResult.length; i++){
                if(path.equals(contentResult[i])){
            pathList.add(path);
                    break;
                }
            }
        }
        }else{
            while (results.next()) {
            String path = results.getString(DatabaseConstants.PATH_FIELD);
            pathList.add(path);
        }
        }

        String[] paths = (String[])pathList.toArray(new String[pathList.size()]);

        return new CollectionImpl(paths);
    }

    private Collection fillCommentsCollection(ResultSet results, Connection conn)
            throws SQLException {

        // SQL query should restrun a list of DatabaseConstants.COMMENT_ID_FIELDs.
        // resultArtifact contains a String[] of URLs for comments.

        // URL for a comment <resource_path>?comment<comment_id>
        // e.g. /p1/r1?comment12

        List commentPathList = new ArrayList();

        while (results.next()) {

            long commentID = results.getLong(DatabaseConstants.COMMENT_ID_FIELD);

            String selectResourcePath =
                    "SELECT R.PATH FROM REG_RESOURCE R, REG_COMMENT C WHERE R.RID=C.RID AND C.COMMENT_ID=?";
            PreparedStatement s1 = conn.prepareStatement(selectResourcePath);
            s1.setLong(1, commentID);

            String path = "";
            ResultSet result = s1.executeQuery();
            if (result.next()) {
                path = result.getString(DatabaseConstants.PATH_FIELD);
            }

            String commentPath = path + RegistryConstants.URL_SEPARATOR + "comments:" + commentID;
            commentPathList.add(commentPath);
        }

        String[] commentPaths =
                (String[])commentPathList.toArray(new String[commentPathList.size()]);

        return new CollectionImpl(commentPaths);
    }

    private Collection fillRatingsCollection(ResultSet results, Connection conn)
            throws SQLException {

        // SQL query should restrun a list of DatabaseConstants.RATING_ID_FIELDs
        // resultArtifact contains a String[] of URLs for ratings.

        // URL for a comment <resource_path>;ratings:<userName>
        // e.g. /p1/r1;ratings:ruwan

        List ratingPathList = new ArrayList();

        while (results.next()) {

            long ratingID = results.getLong(DatabaseConstants.RATING_ID_FIELD);

            RatingsDAO ratingsDAO = new RatingsDAO();
            RatingDO ratingDAO = ratingsDAO.getRating(ratingID);

            String ratingPath =
                    ratingDAO.getResourcePath() + RegistryConstants.URL_SEPARATOR + "ratings:" +
                    ratingDAO.getRatedUserName();
            ratingPathList.add(ratingPath);
        }

        String[] ratingPaths =
                (String[])ratingPathList.toArray(new String[ratingPathList.size()]);

        return new CollectionImpl(ratingPaths);
    }

    private Collection fillTagsCollection(ResultSet results)
            throws SQLException, RegistryException {

        // URL for a tag /p1/r1;tags:tagName:userName

        List tagPathList = new ArrayList();
        while (results.next()) {

            long taggingID = results.getLong(DatabaseConstants.TAGGING_ID_FIELD);

            TagsDAO tagsDAO = new TagsDAO();
            TaggingDO taggingDO = tagsDAO.getTagging(taggingID);

            //String userName = results.getString(DatabaseConstants.USER_ID_FIELD);
            //long artifactID = results.getLong(DatabaseConstants.ARTIFACT_ID_FIELD);
            //long tagID = results.getLong(DatabaseConstants.TAG_ID_FIELD);
            //
            //String artifactSQL = "SELECT PATH FROM ARTIFACTS WHERE AID=?";
            //PreparedStatement s1 = conn.prepareStatement(artifactSQL);
            //s1.setLong(1, artifactID);
            //ResultSet result1 = s1.executeQuery();
            //
            //String taggedPath = "";
            //if (result1.next()) {
            //    taggedPath = result1.getString(DatabaseConstants.PATH_FIELD);
            //}
            //
            //String tagSQL = "SELECT TAG_NAME FROM TAGS WHERE TAG_ID=?";
            //PreparedStatement s2 = conn.prepareStatement(tagSQL);
            //s2.setLong(1, tagID);
            //ResultSet result2 = s2.executeQuery();
            //
            //String tag = "";
            //if (result2.next()) {
            //    tag = result2.getString(DatabaseConstants.TAG_NAME_FIELD);
            //}

            String tagPath = taggingDO.getResourcePath() + RegistryConstants.URL_SEPARATOR +
                             "tags:" + taggingDO.getTagName() + ":" + taggingDO.getTaggedUserName();
            tagPathList.add(tagPath);
        }

        String[] tagPaths = (String[])tagPathList.toArray(new String[tagPathList.size()]);

        return new CollectionImpl(tagPaths);
    }
}
