/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.realm;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.authenticator.Authenticator;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.users.*;
import org.wso2.registry.users.def.DefaultRealm;
import org.wso2.registry.users.def.DefaultRealmConfig;
import org.wso2.registry.utils.AuthorizationUtils;

import javax.sql.DataSource;

/**
 * This class makes sure that the user store of the registry is always in consistent state. It wraps
 * the given realm and peforms validations for all required actions before delegating the
 * requests to the wrapped realm.
 *
 * Realm is only used for getting other user store related classes. Therefore, registry provides
 * a wrappers for all such classes whereever a validation is necessary.
 */
public class RegistryRealm implements UserRealm {

    private static final Log log = LogFactory.getLog(RegistryRealm.class);

    private Authenticator authenticator;
    private UserStoreReader userStoreReader;
    private UserStoreAdmin userStoreAdmin;
    private Authorizer authorizer;
    private AccessControlAdmin accessControlAdmin;

    private UserRealm coreRealm;

    /**
     * This contructor is provided for extending the RegistryRealm.
     */
    public RegistryRealm() {}

    /**
     * Creates a default realm from the given datasource.
     *
     * @param dataSource
     * @throws RegistryException
     */
    public RegistryRealm(DataSource dataSource) throws RegistryException {
        initialize(dataSource);
    }

    public RegistryRealm(UserRealm userRealm) throws RegistryException {
        initialize(userRealm);
    }

    protected void initialize(DataSource dataSource) throws RegistryException {

        DefaultRealm defaultRealm;

        try {

            DefaultRealmConfig config = new DefaultRealmConfig();
            config.setDataSource(dataSource);

            defaultRealm = new DefaultRealm();
            defaultRealm.init(config);

        } catch (UserStoreException e) {
            String msg = "Could not create a user manager realm using the given data source. " +
                    "Caused by: " + e.getMessage();
            log.fatal(msg, e);
            throw new RegistryException(msg, e);
        }

        initialize(defaultRealm);
    }

    private void initialize(UserRealm userRealm) throws RegistryException {

        try {
            this.coreRealm = userRealm;
            this.authenticator = coreRealm.getAuthenticator();
            this.userStoreReader = coreRealm.getUserStoreReader();
            this.userStoreAdmin = new RegistryUserStoreAdmin(coreRealm.getUserStoreAdmin());
            this.authorizer = coreRealm.getAuthorizer();
            this.accessControlAdmin =
                    new RegistryAccessControlAdmin(coreRealm.getAccessControlAdmin());

            AuthorizationUtils.populateUserStore(this);

        } catch (UserStoreException e) {
            String msg = "Failed to initialize the registry user store. " + e.getMessage();
            log.fatal(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public Object getRealmConfiguration() throws UserStoreException {
        return coreRealm.getRealmConfiguration();
    }

    public void init(Object configBean) throws UserStoreException {
        coreRealm.init(configBean);
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException {
        return userStoreAdmin;
    }

    public AccessControlAdmin getAccessControlAdmin() throws UserStoreException {
        return accessControlAdmin;
    }

    public Authenticator getAuthenticator() throws UserStoreException {
        return authenticator;
    }

    public Authorizer getAuthorizer() throws UserStoreException {
        return authorizer;
    }

    public UserStoreReader getUserStoreReader() throws UserStoreException {
        return userStoreReader;
    }
}
