/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.utils.creators;

import javax.sql.DataSource;
import java.sql.Statement;
import java.sql.SQLException;

public class HSQLDatabaseCreator extends DatabaseCreator {

    private static final String resourceTable =
            "CREATE TABLE REG_RESOURCE (" +
                    "RID VARCHAR (50)," +
                    "PATH VARCHAR (2000) NOT NULL," +
                    "MEDIA_TYPE VARCHAR (500)," +
                    "COLLECTION INTEGER NOT NULL," +
                    "CREATOR VARCHAR (500)," +
                    "CREATED_TIME TIMESTAMP," +
                    "LAST_UPDATOR VARCHAR (500)," +
                    "LAST_UPDATED_TIME TIMESTAMP," +
                    "DESCRIPTION VARCHAR (10000)," +
                    "CONTENT_ID VARCHAR (50)," +
                    "EQUIVALENT_VERSION INTEGER NOT NULL," +
                    "ASSOCIATED_SNAPSHOT_ID INTEGER NOT NULL," +
                    "PRIMARY KEY (RID)," +
                    "FOREIGN KEY (CONTENT_ID) REFERENCES REG_CONTENT (CONTENT_ID)," +
                    "UNIQUE(PATH));";

    private static final String contentTable =
            "CREATE TABLE REG_CONTENT (" +
                    "CONTENT_ID VARCHAR (50)," +
                    "CONTENT_DATA BINARY," +
                    "PRIMARY KEY (CONTENT_ID));";

    // Table to store parent child relationships in current versions of resources. Collection
    // which is referred in PARENT_RID is not allowed to delete until the relationship is
    // explicitly removed from this table. This is to ensure that collections are not deleted
    // till all its children are removed. If a child resource (which has no children) is deleted
    // its parent child relationship is automatically deleted.

    private static final String dependencyTable =
            "CREATE TABLE REG_DEPENDENCY (" +
                    "DEPENDENCY_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "PARENT_RID VARCHAR (50) NOT NULL," +
                    "CHILD_RID VARCHAR (50) NOT NULL," +
                    "PRIMARY KEY (DEPENDENCY_ID)," +
                    "UNIQUE (PARENT_RID, CHILD_RID)," +
                    "FOREIGN KEY (PARENT_RID) REFERENCES REG_RESOURCE (RID) ON DELETE CASCADE," +
                    "FOREIGN KEY (CHILD_RID) REFERENCES REG_RESOURCE (RID) ON DELETE CASCADE);";

    // We can't make (RID, NAME, PROPERTY_VALUE) a unique tuple as resource may have a property with
    // repeating values. e.g.p1 -> v1, v2, v3, v3, v4
    private static final String propertyTable =
            "CREATE TABLE REG_PROPERTY (" +
                    "PROPERTY_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "RID VARCHAR (50) NOT NULL," +
                    "NAME VARCHAR (100) NOT NULL," +
                    "PROPERTY_VALUE VARCHAR (500)," +
                    "PRIMARY KEY (PROPERTY_ID)," +
                    "FOREIGN KEY (RID) REFERENCES REG_RESOURCE (RID) ON DELETE CASCADE);";

    private static final String associationTable =
            "CREATE TABLE REG_ASSOCIATION (" +
                    "ASSOCIATION_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "SOURCEPATH VARCHAR (2000) NOT NULL," +
                    "TARGETPATH VARCHAR (2000) NOT NULL," +
                    "ASSOCIATION_TYPE VARCHAR (2000) NOT NULL," +
                    "PRIMARY KEY (ASSOCIATION_ID)," +
                    "UNIQUE (SOURCEPATH, TARGETPATH, ASSOCIATION_TYPE))";

    // Note that comments, tags and ratings are not versioned. Therefore, when a resource is deleted
    // its comments, etc should not be deleted. When a deleted resource is restored, its
    // comments, etc are reassociated with it.

    private static final String tagTable =
            "CREATE TABLE REG_TAG (" +
                    "TAG_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "TAG_NAME VARCHAR (500) NOT NULL," +
                    "RID VARCHAR (50) NOT NULL," +
                    "USER_ID VARCHAR (20) NOT NULL," +
                    "TAGGED_TIME TIMESTAMP NOT NULL," +
                    "PRIMARY KEY (TAG_ID)," +
                    "UNIQUE (TAG_NAME, RID, USER_ID));";

    private static final String commentTable =
            "CREATE TABLE REG_COMMENT (" +
                    "COMMENT_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "RID VARCHAR (50) NOT NULL," +
                    "USER_ID VARCHAR (20) NOT NULL," +
                    "COMMENT_TEXT VARCHAR (500) NOT NULL," +
                    "COMMENTED_TIME TIMESTAMP NOT NULL," +
                    "PRIMARY KEY (COMMENT_ID));";

    private static final String ratingTable =
            "CREATE TABLE REG_RATING (" +
                    "RATING_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "RID VARCHAR (50) NOT NULL," +
                    "USER_ID VARCHAR (20) NOT NULL," +
                    "RATING INTEGER NOT NULL," +
                    "RATED_TIME TIMESTAMP NOT NULL," +
                    "PRIMARY KEY (RATING_ID));";

    private static final String logTable =
            "CREATE TABLE REG_LOG (" +
                    "LOG_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "PATH VARCHAR (2000)," +
                    "USER_ID VARCHAR (20) NOT NULL," +
                    "LOGGED_TIME TIMESTAMP NOT NULL," +
                    "ACTION INTEGER NOT NULL," +
                    "ACTION_DATA VARCHAR (500)," +
                    "PRIMARY KEY (LOG_ID));";

    //////////////////////////////////////////////////////////////////////////////////
    // Tables related to old versions of resources and metadata                     //
    //////////////////////////////////////////////////////////////////////////////////

    private static final String resourceVersionTable =
            "CREATE TABLE REG_RESOURCE_VERSION (" +
                    "RESOURCE_VERSION_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "RID VARCHAR (50)," +
                    "VERSION INTEGER NOT NULL," +
                    "PATH VARCHAR (2000) NOT NULL," +
                    "MEDIA_TYPE VARCHAR (500)," +
                    "COLLECTION INTEGER NOT NULL," +
                    "CREATOR VARCHAR (500)," +
                    "CREATED_TIME TIMESTAMP," +
                    "LAST_UPDATOR VARCHAR (500)," +
                    "LAST_UPDATED_TIME TIMESTAMP," +
                    "DESCRIPTION VARCHAR (10000)," +
                    "CONTENT_ID VARCHAR (50)," +
                    "ASSOCIATED_SNAPSHOT_ID INTEGER NOT NULL," +
                    "FOREIGN KEY (CONTENT_ID) REFERENCES REG_CONTENT_VERSION (CONTENT_VERSION_ID)," +
                    "PRIMARY KEY (RESOURCE_VERSION_ID)," +
                    "UNIQUE (RID, VERSION))";

    private static final String dependencyVersionTable =
            "CREATE TABLE REG_DEPENDENCY_VERSION (" +
                    "DEPENDENCY_VERSION_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "PARENT_RID VARCHAR (50) NOT NULL," +
                    "PARENT_VERSION INTEGER NOT NULL," +
                    "CHILD_RID VARCHAR (50) NOT NULL," +
                    "PRIMARY KEY (DEPENDENCY_VERSION_ID)," +
                    "UNIQUE (PARENT_RID, PARENT_VERSION, CHILD_RID))";

    private static final String contentVersionTable =
            "CREATE TABLE REG_CONTENT_VERSION (" +
                    "CONTENT_VERSION_ID VARCHAR (50)," +
                    "CONTENT_DATA BINARY," +
                    "PRIMARY KEY (CONTENT_VERSION_ID));";

    private static final String propertyVersionTable =
            "CREATE TABLE REG_PROPERTY_VERSION (" +
                    "PROPERTY_VERSION_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "RID VARCHAR (50) NOT NULL," +
                    "VERSION INTEGER NOT NULL," +
                    "NAME VARCHAR (100) NOT NULL," +
                    "PROPERTY_VALUE VARCHAR (500)," +
                    "PRIMARY KEY (PROPERTY_VERSION_ID)," +
                    "FOREIGN KEY (RID, VERSION) REFERENCES REG_RESOURCE_VERSION (RID, VERSION));";

    //////////////////////////////////////////////////////////////////////////////////
    // Tables related to snapshot management                                        //
    //////////////////////////////////////////////////////////////////////////////////

    // UUID is used for snapshot ID, so that snapshots can be achieved and reloaded without
    // colliding with existing snapshots.

    private static final String snapshotTable =
            "CREATE TABLE REG_SNAPSHOT (" +
                    "SNAPSHOT_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "ROOT_ID VARCHAR (50) NOT NULL," +
                    "PRIMARY KEY (SNAPSHOT_ID)," +
                    "UNIQUE (SNAPSHOT_ID, ROOT_ID));";

    private static final String snapshotRootIDIndex =
            "CREATE INDEX INDEX_SNAPSHOT_ROOT_ID ON REG_SNAPSHOT (ROOT_ID)";

    private static final String snapshotResourceVersionTable =
            "CREATE TABLE REG_SNAPSHOT_RESOURCE_VERSION (" +
                    "SRV_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
                    "SNAPSHOT_ID INTEGER NOT NULL," +
                    "RID VARCHAR (50) NOT NULL," +
                    "VERSION INTEGER NOT NULL," +
                    "PRIMARY KEY (SRV_ID)," +
                    "UNIQUE (SNAPSHOT_ID, RID, VERSION));";

    ////////////////////////////////////////////////////////////////////////////////////
    // User manager tables                                                            //
    ////////////////////////////////////////////////////////////////////////////////////

    public static String userTable = "CREATE TABLE UM_USERS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY," + "USER_NAME VARCHAR(255) NOT NULL,"
            + "USER_PASSWORD VARCHAR(255) NOT NULL, " + "PRIMARY KEY (ID), " + "UNIQUE(USER_NAME))";

    public static String userAttributeTable = "CREATE TABLE UM_USER_ATTRIBUTES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY," + "ATTR_NAME VARCHAR(255) NOT NULL,"
            + "ATTR_VALUE VARCHAR(255), " + "USER_ID INTEGER, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static String roleTable = "CREATE TABLE UM_ROLES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ROLE_NAME VARCHAR(255) NOT NULL, "
            + "PRIMARY KEY (ID), " + "UNIQUE(ROLE_NAME))";

    public static String roleAttributeTable = "CREATE TABLE UM_ROLE_ATTRIBUTES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ATTR_NAME VARCHAR(255) NOT NULL, "
            + "ATTR_VALUE VARCHAR(255), " + "ROLE_ID INTEGER, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static String permissionTable = "CREATE TABLE UM_PERMISSIONS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, "
            + "RESOURCE_ID VARCHAR(255) NOT NULL, " + "ACTION VARCHAR(255) NOT NULL, "
            + "PRIMARY KEY (ID))";

    public static String permissionResourceIDActionIndex =
            "CREATE INDEX INDEX_UM_PERMISSIONS_RESOURCE_ID_ACTION " +
                    "ON UM_PERMISSIONS (RESOURCE_ID, ACTION)";

    public static String rolePermissionTable = "CREATE TABLE UM_ROLE_PERMISSIONS ( "
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "PERMISSION_ID INTEGER NOT NULL, "
            + "ROLE_ID INTEGER NOT NULL, " + "IS_ALLOWED SMALLINT NOT NULL, "
            + "UNIQUE (PERMISSION_ID, ROLE_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE  CASCADE, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static String userPermissionTable = "CREATE TABLE UM_USER_PERMISSIONS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "PERMISSION_ID INTEGER NOT NULL, "
            + "USER_ID INTEGER NOT NULL, " + "IS_ALLOWED SMALLINT NOT NULL, "
            + "UNIQUE (PERMISSION_ID, USER_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static String userRolesTable = "CREATE TABLE UM_USER_ROLES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ROLE_ID INTEGER NOT NULL, "
            + "USER_ID INTEGER NOT NULL, " + "UNIQUE (USER_ID, ROLE_ID), "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public HSQLDatabaseCreator(DataSource dataSource) {
        super(dataSource);
    }

    public String getResourceTable() {
        return resourceTable;
    }

    public String getContentTable() {
        return contentTable;
    }

    public String getDependencyTable() {
        return dependencyTable;
    }

    public String getPropertyTable() {
        return propertyTable;
    }

    public String getAssociationTable() {
        return associationTable;
    }

    public String getTagTable() {
        return tagTable;
    }

    public String getCommentTable() {
        return commentTable;
    }

    public String getRatingTable() {
        return ratingTable;
    }

    public String getLogTable() {
        return logTable;
    }

    public String getResourceVersionTable() {
        return resourceVersionTable;
    }

    public String getDependencyVersionTable() {
        return dependencyVersionTable;
    }

    public String getContentVersionTable() {
        return contentVersionTable;
    }

    public String getPropertyVersionTable() {
        return propertyVersionTable;
    }

    public String getSnapshotTable() {
        return snapshotTable;
    }

    public String getSnapshotResourceVersionTable() {
        return snapshotResourceVersionTable;
    }

    public String getUserTable() {
        return userTable;
    }

    public String getUserAttributeTable() {
        return userAttributeTable;
    }

    public String getRoleTable() {
        return roleTable;
    }

    public String getRoleAttributeTable() {
        return roleAttributeTable;
    }

    public String getPermissionTable() {
        return permissionTable;
    }

    public String getRolePermissionTable() {
        return rolePermissionTable;
    }

    public String getUserPermissionTable() {
        return userPermissionTable;
    }

    public String getUserRolesTable() {
        return userRolesTable;
    }

    public String getSnapshotRootIDIndex() {
        return snapshotRootIDIndex;
    }

    public void executeResourceStoreDatabaseSpecificStatements(Statement s) throws SQLException {
    }

    public void executeUserStoreDatabaseSpecificStatements(Statement s) throws SQLException {
        s.executeUpdate(permissionResourceIDActionIndex);
    }
}
