/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.users.accesscontrol;

import org.wso2.registry.i18n.Messages;
import org.wso2.registry.users.Authorizer;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.UserStoreReader;

import java.util.Map;

/**
 * @see org.wso2.usermanager.UserStoreReader
 */
public class ACLUserStoreReader implements UserStoreReader {

    private Authorizer authorizer = null;

    private UserStoreReader usReader = null;

    protected AuthorizingRealmConfig config = null;

    protected boolean isAdmin = false;

    public ACLUserStoreReader(Authorizer athzr, UserStoreReader reader,
            AuthorizingRealmConfig authConfig) {
        this.authorizer = athzr;
        this.config = authConfig;
        this.usReader = reader;

        if (config.isEnableAdminBehavior()
                && ACLAdminChecker.isAdminUser(config.getAuthenticatedUserName())) {
            isAdmin = true;
        }

    }

    public String[] getAllUserNames() throws UserStoreException {
        doAuthorizationToReadUser();
        return usReader.getAllUserNames();
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        doAuthorizationToReadUser();
        return usReader.getUserNamesWithPropertyValue(propertyName, propetyValue);
    }

    public Map getUserProperties(String userName) throws UserStoreException {
        if (config.getAuthenticatedUserName().equals(userName) && config.isCurrentUserReadable()) {
            // do nothing authorized
        } else {
            doAuthorizationToReadUser();
        }
        return usReader.getUserProperties(userName);
    }

    public String[] getUserPropertyNames() throws UserStoreException {
        doAuthorizationToReadUser();
        return usReader.getUserPropertyNames();
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        doAuthorizationToReadUser();
        return usReader.isExistingUser(userName);
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        doAuthorizationToReadRole();
        return usReader.isExistingUser(roleName);
    }

    public String[] getAllRoleNames() throws UserStoreException {
        doAuthorizationToReadRole();
        return usReader.getAllRoleNames();
    }

    public Map getRoleProperties(String roleName) throws UserStoreException {
        doAuthorizationToReadRole();
        return usReader.getRoleProperties(roleName);
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        doAuthorizationToReadRole();
        return usReader.getUserRoles(userName);
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        doAuthorizationToReadRole();
        return usReader.getUsersInRole(roleName);
    }

    protected void doAuthorizationToReadRole() throws UserStoreException {

        if (config.isEnableAdminBehavior() && isAdmin) {
            // do nothing user is authenticated
        } else if (!authorizer.isUserAuthorized(config.getAuthenticatedUserName(),
                AccessControlConstants.ROLE_RESOURCE, AccessControlConstants.READ)) {
            throw new UnauthorizedException(Messages.getMessage("unAuthorized", new String[] {
                    AccessControlConstants.ROLE_RESOURCE, AccessControlConstants.READ }));
        }
    }

    protected void doAuthorizationToReadUser() throws UserStoreException {

        if (config.isEnableAdminBehavior() && isAdmin) {
            // do nothing user is authenticated
        } else if (!authorizer.isUserAuthorized(config.getAuthenticatedUserName(),
                AccessControlConstants.USER_RESOURCE, AccessControlConstants.READ)) {
            throw new UnauthorizedException(Messages.getMessage("unAuthorized", new String[] {
                    AccessControlConstants.USER_RESOURCE, AccessControlConstants.READ }));
        }
    }

}
