/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.app;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.wso2.registry.RegistryConstants;

/**
 * These tests use a raw HTTP client in order to exercise the APP implementation of our Registry
 * server.  We do this to make sure that the protocol exactly matches what we expect; using the
 * RemoteRegistry (as the JettyBasedServerTest does) is a "black box" test which only proves the API
 * works.  This test confirms wire-level behavior.
 */
public class APPProtocolTest extends AbstractAPPTest   {
    HttpClient client = new HttpClient();

    protected void setUp() throws Exception {
//        clientPort = 8082; // tcpmon
        super.setUp();
    }

    public String buildURL(String path) {
        if (path.startsWith(("/"))) path = path.substring(1);
        return baseURL + "atom/" + path;
    }

    /**
     * Run through a series of actions against a clean registry, and confirm the results match our
     * expectations.
     *
     * @throws Exception if there's a problem
     */
    public void testProtocol() throws Exception {
        client.getParams().setAuthenticationPreemptive(true);
        Credentials defaultcreds = new UsernamePasswordCredentials("admin", "admin");
        client.getState().setCredentials(new AuthScope("localhost", clientPort, AuthScope.ANY_REALM),
                                         defaultcreds);

        // Note - every URL below is rooted at our Registry's atom root

        // GET the root - empty dir
        HttpMethod method = new GetMethod(buildURL(""));
        int responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);

        method.getResponseBody();
        method.releaseConnection();

        // GET /d1 - 404
        method = new GetMethod(buildURL("d1"));
        responseCode = client.executeMethod(method);
        assertEquals(404, responseCode); // Nothing there yet!

        // GET /foo;tags - 404
        // Make sure metadata indicator doesn't change error code
        method = new GetMethod(buildURL("foo" + RegistryConstants.URL_SEPARATOR + "tags"));
        responseCode = client.executeMethod(method);
        assertEquals(404, responseCode);
        method.releaseConnection();

        // POST / with d1 content
        // TODO - fix this up.
        String atomContent =
                "<feed xmlns=\"http://www.w3.org/2005/Atom\" xmlns:ns=\"tag:wso2.org,2008:foo\"><summary type=\"text\" /><author><name>system</name></author><ns:properties /></feed>";
        String location = doPost(buildURL(""), atomContent, "d1", "application/atom+xml", 201);
        assertNotNull(location);
        //assertEquals("correct header value", location);

        // GET /d1 - empty dir (but not 404!)
        method = new GetMethod(buildURL("d1"));
        responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);
        method.releaseConnection();

        // GET /d1;tags - empty tags (or 404?)
        method = new GetMethod(buildURL("d1;tags"));
        responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);
        method.releaseConnection();

        // POST / with no slug - text file
        String textContent = "hi there here's some text";
        location = doPost(buildURL(""), textContent, null, null, 201);
        assertNotNull(location);

        // GET of the returned location should return something reasonable...
        method = new GetMethod(location);
        responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);
        // We should have here an Atom entry pointing to the actual media.  Parse
        // that out and get the media URL, then GET that and make sure the content
        // is kosher.

        // POST /d1 with bad slug "&test?"
        // Should return /d1/test (sanitized name)
        textContent = "hi there here's some text";
        location = doPost(buildURL("d1"), textContent, ";test?", null, 201);
        assertNotNull(location);
        assertTrue(location.endsWith("d1/test"));

        // GET /d1/test - content
        method = new GetMethod(location);
        responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);
        String responseBody = method.getResponseBodyAsString();
        assertEquals(textContent, responseBody);

        // POST /d1 with existing slug "test"
        location = doPost(buildURL("d1"), textContent, "test", null, 201);
        assertNotNull(location);
        // Should return /d1/test2 (or some sort of edited name)
        assertFalse(location.endsWith("d1/test"));

        // Tags

        // GET /d1/test;tags - empty tags (or 404?)

        // POST /d1/test;tags tag=blue
        doPost(buildURL("/d1/test;tags"), "blue", null, null, 200);

        // GET /d1/test;tags - "blue"
        method = new GetMethod(buildURL("/d1/test;tags"));
        responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);
        responseBody = method.getResponseBodyAsString();
        assertTrue(responseBody.indexOf("blue") > -1);

        // Ratings

        // Comments
    }

    private String doPost(String url,
                          String content,
                          String slug,
                          String contentType,
                          int expectedResult)
            throws Exception {
        if (contentType == null) contentType = "text/plain";
        
        PostMethod writeMethod = new PostMethod(url);
        if (slug != null)
            writeMethod.addRequestHeader("Slug", slug);
        StringRequestEntity myRequest = new StringRequestEntity(content, contentType, null);
        writeMethod.setRequestEntity(myRequest);
        int responseCode = client.executeMethod(writeMethod);
        assertEquals(expectedResult, responseCode);

        // Get the location header
        String location = writeMethod.getResponseHeader("Location").getValue();
        assertNotNull(location);

        writeMethod.releaseConnection();

        // Parse out result path
        return location;
    }
}
