/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.web.processors.*;
import org.wso2.registry.web.utils.CommonUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServlet;
import javax.servlet.ServletException;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.Map;
import java.util.HashMap;

/**
 * This servlet processes the commands (mostly AJAX commands) send from the web UI. Ending
 * section of the URL identifies the command to execute.
 */
public class CommandServlet extends HttpServlet {

    private static final Log log = LogFactory.getLog(CommandServlet.class);

    private String contextRoot;

    private int serverPrefixLength;

    private Map <String, UIProcessor> getProcessorMap = new HashMap <String, UIProcessor> ();
    private Map <String, UIProcessor> postProcessorMap = new HashMap <String, UIProcessor> ();

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();

        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;

        serverPrefixLength = contextRoot.length() +
                RegistryConstants.PATH_SEPARATOR.length() + UIConstants.SYSTEM_PATH.length();

        initializeCommands();
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        initContextRoot(request);

        String uri = URLDecoder.decode(request.getRequestURI(), "UTF-8");
        String command = uri.substring(serverPrefixLength, uri.length());

        UIProcessor uiProcessor = getProcessorMap.get(command);
        if (uiProcessor == null) {
            String msg = "Unknown URL pattern: " + command +
                    ".\nURL processor is not installed for this URL.";
            log.error(msg);
            CommonUtil.sendErrorContent(response, msg);
        } else {
            uiProcessor.process(request, response);
        }
    }


    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        initContextRoot(request);

        String uri = URLDecoder.decode(request.getRequestURI(), "UTF-8");
        String command = uri.substring(serverPrefixLength, uri.length());
        
        UIProcessor uiProcessor = postProcessorMap.get(command);
        if (uiProcessor == null) {
            String msg = "Unknown URL pattern: " + command +
                    ".\nURL processor is not installed for this URL.";
            log.error(msg);
            CommonUtil.sendErrorContent(response, msg);
        } else {
            uiProcessor.process(request, response);
        }
    }

    private void initializeCommands() {

        getProcessorMap.put("/signout", new SignOutProcessor());
        getProcessorMap.put("/associationTree", new AssociationTreeProcessor());
        getProcessorMap.put("/getAnnouncement", new GetAnnouncementProcessor());
        getProcessorMap.put("/resourceTree", new GetResourceTreeProcessor());
        getProcessorMap.put("/getTagCloud", new GetTagCloudProcessor());
        getProcessorMap.put("/getInitialRating", new GetInitialRatingProcessor());
        getProcessorMap.put("/getRating", new GetRatingProcessor());
        getProcessorMap.put("/getDescription", new GetDescriptionProcessor());
        getProcessorMap.put("/deleteResource", new DeleteResourceProcessor());
        getProcessorMap.put("/getResourceMediaTypes", new GetResourceMediaTypesProcessor());
        getProcessorMap.put("/restore", new RestoreResourceProcessor());
        getProcessorMap.put("/removeUserFromRole", new RemoveUserRoleProcessor());
        getProcessorMap.put("/removeUser", new RemoveUserProcessor());
        getProcessorMap.put("/removeRole", new RemoveRoleProcessor());
        getProcessorMap.put("/getActivity", new GetActivityProcessor());
        getProcessorMap.put("/advancedSearch", new AdvancedSearchProcessor());
        getProcessorMap.put("/toggleSysProps", new ToggleSystemPropertiesProcessor());
        getProcessorMap.put("/getResourceCreationUI", new ResourceCreationUIProcessor());

        postProcessorMap.put("/signin", new SignInProcessor());
        postProcessorMap.put("/addUser", new AddUserProcessor());
        postProcessorMap.put("/addRole", new AddRoleProcessor());
        postProcessorMap.put("/applyTag", new ApplyTagProcessor());
        postProcessorMap.put("/createVersion", new CreateVersionProcessor());
        postProcessorMap.put("/authorize", new UserAuthorizationProcessor());
        postProcessorMap.put("/authorize/mass", new MassUserAuthorizationProcessor());
        postProcessorMap.put("/authorizeRole", new RoleAuthorizationProcessor());
        postProcessorMap.put("/authorizeRoles", new MassRoleAuthorizationProcessor());
        postProcessorMap.put("/setDescription", new SetDescriptionProcessor());
        postProcessorMap.put("/setRating", new SetRatingProcessor());
        postProcessorMap.put("/renameResource", new RenameResourceProcessor());
        postProcessorMap.put("/copyResource", new CopyResourceProcessor());
        postProcessorMap.put("/moveResource", new MoveResourceProcessor());
        postProcessorMap.put("/givePage", new GivePageProcessor());
        postProcessorMap.put("/addComment", new AddCommentProcessor());
        postProcessorMap.put("/addAspect", new AddAspectProcessor());
        postProcessorMap.put("/setProperty", new SetPropertyProcessor());
        postProcessorMap.put("/updateProperty", new UpdatePropertyProcessor());
        postProcessorMap.put("/addRemoveAssociation", new AddRemoveAssociationProcessor());
        postProcessorMap.put("/invokeAspect", new InvokeAspectProcessor());
        postProcessorMap.put("/removeProperty", new RemovePropertyProcessor());
        postProcessorMap.put("/addCollection", new AddCollectionProcessor());
        postProcessorMap.put("/fetchResource", new FetchResourceProcessor());
        postProcessorMap.put("/addResource", new AddResourceProcessor());
        postProcessorMap.put("/addTextResource", new AddTextResourceProcessor());
        postProcessorMap.put("/saveFriendlyName", new SaveFriendlyNameProcessor());
        postProcessorMap.put("/saveNewPassword", new SaveNewPasswordProcessor());
        postProcessorMap.put("/addUserToRole", new AddUserRoleProcessor());
    }
}
