/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.exceptions.RegistryException;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

public class MediaTypesReader {

    public static final String RESOURCE_MEDIA_TYPE_MAPPINGS_FILE = "/WEB-INF/mime.types";
    public static final String COLLECTION_MEDIA_TYPES_FILE = "/WEB-INF/collection-media-types.conf";

    public static String getResourceMediaTypeMappings(HttpServletRequest request) throws RegistryException {

        InputStream input =
                request.getSession().getServletContext().getResourceAsStream(RESOURCE_MEDIA_TYPE_MAPPINGS_FILE);
        BufferedReader reader = new BufferedReader(new InputStreamReader(input));

        String mediaTypeString = null;
        try {
            while(reader.ready()) {
                String mediaTypeData = reader.readLine().trim();
                if (mediaTypeData.startsWith("#")) {
                    // ignore the comments
                    continue;
                }

                if (mediaTypeData.length() == 0) {
                    // ignore the blank lines
                    continue;
                }

                // mime.type file delimits mediatypes:extensions by tabs. if there is no
                // extension associated with a media type, there are no tabs in the line. so we
                // don't need such lines.
                if (mediaTypeData.indexOf("\t") > 0) {

                    String[] parts = mediaTypeData.split("\t+");
                    if (parts.length == 2 && parts[0].length() > 0 && parts[1].length() > 0) {
                        
                        // there can multiple extensions associated with a single media type. in
                        // that case, extensions are delimited by a space.
                        String[] exts = parts[1].trim().split(" ");
                        for (int i = 0; i < exts.length; i++) {

                            if (exts[i].length() > 0) {
                                String mediaTypeMapping = exts[i] + ":" + parts[0];

                                if (mediaTypeString == null) {
                                    mediaTypeString = mediaTypeMapping;
                                } else {
                                    mediaTypeString = mediaTypeString + "," + mediaTypeMapping;
                                }
                            }
                        }
                    }
                }
            }

        } catch (IOException e) {
            String msg = "Could not read the media type mappings file from the location: ";
            throw new RegistryException(msg);

        } finally {
            try {
                reader.close();
                input.close();
            } catch (IOException ignore) {
            }
        }

        return mediaTypeString;
    }

    public static List getCollectionMediaTypes(HttpServletRequest request)
            throws RegistryException {

        InputStream input =
                request.getSession().getServletContext().getResourceAsStream(COLLECTION_MEDIA_TYPES_FILE);
        BufferedReader reader = new BufferedReader(new InputStreamReader(input));

        List collectionMediaTypes = new ArrayList();
        try {
            while(reader.ready()) {
                String mediaType = reader.readLine();
                collectionMediaTypes.add(mediaType);
            }

        } catch (IOException e) {
            String msg = "Could not read the media type mappings file from the location: " + COLLECTION_MEDIA_TYPES_FILE;
            throw new RegistryException(msg);

        } finally {
            try {
                reader.close();
                input.close();
            } catch (IOException ignore) {
            }
        }

        return collectionMediaTypes;
    }
}
