/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.*;
import org.wso2.registry.Collection;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.web.actions.utils.AdvancedResourceQuery;
import org.wso2.registry.web.actions.utils.ResourceData;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.text.DateFormat;
import java.text.ParseException;

public class AdvancedSearchAction extends AbstractRegistryAction {

    private Log log = LogFactory.getLog(AdvancedSearchAction.class);

    private String resourceName;
    private String authorName;
    private String updaterName;
    private String createdAfter;
    private String createdBefore;
    private String updatedAfter;
    private String updatedBefore;
    private String tags;
    private String commentWords;
    private String propertyName;
    private String propertyValue;
    private String content;
    private String resultType = "Resources";
    private String[] childPaths;

    private List resourceDataList = new ArrayList();

    public void execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        Registry registry = getRegistry();

        if (resultType.equals("Resources")) {

            AdvancedResourceQuery query = new AdvancedResourceQuery();
            query.setResourceName(resourceName);
            query.setAuthorName(authorName);
            query.setUpdaterName(updaterName);
            query.setCreatedAfter(computeDate(createdAfter));
            query.setCreatedBefore(computeDate(createdBefore));
            query.setUpdatedAfter(computeDate(updatedAfter));
            query.setUpdatedBefore(computeDate(updatedBefore));
            query.setCommentWords(commentWords);
            query.setTags(tags);
            query.setPropertyName(propertyName);
            query.setPropertyValue(propertyValue);
            query.setContent(content);

            Resource qResults = query.execute(registry);

            String[] childPaths = (String[]) qResults.getContent();
            for (String childPath : childPaths) {

                ResourceData resourceData = new ResourceData();
                resourceData.setResourcePath(childPath);

                if (childPath != null) {
                    if (RegistryConstants.ROOT_PATH.equals(childPath)) {
                        resourceData.setName("root");
                    } else {
                        String[] parts = childPath.split(RegistryConstants.PATH_SEPARATOR);
                        resourceData.setName(parts[parts.length - 1]);
                    }
                }

                try {
                    Resource child = registry.get(childPath);

                    resourceData.setResourceType(child instanceof Collection ?
                            ResourceData.COLLECTION : ResourceData.RESOURCE);
                    resourceData.setAuthorUserName(child.getAuthorUserName());
                    resourceData.setDescription(child.getDescription());
                    resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                    resourceData.setCreatedOn(child.getCreatedTime());
                    calculateAverageStars(resourceData);

                    resourceDataList.add(resourceData);

                } catch (AuthorizationFailedException e) {
                    log.info(
                            "AuthorizationFailedException inside execute method of AdvancedSearchAction");
                    // do not show unauthorized resource in search results.
                }
            }
        }
    }
    public String[] getChildPaths() {
        return childPaths;
    }

    public void setChildPaths(String[] childPaths) {
        this.childPaths = childPaths;
    }
    public String getResourceName() {
        return resourceName;
    }

    public void setResourceName(String resourceName) {
        this.resourceName = resourceName;
    }

    public String getAuthorName() {
        return authorName;
    }

    public void setAuthorName(String authorName) {
        this.authorName = authorName;
    }

    public String getUpdaterName() {
        return updaterName;
    }

    public void setUpdaterName(String updaterName) {
        this.updaterName = updaterName;
    }

    public String getCreatedAfter() {
        return createdAfter;
    }

    public void setCreatedAfter(String createdAfter) {
        this.createdAfter = createdAfter;
    }

    public String getCreatedBefore() {
        return createdBefore;
    }

    public void setCreatedBefore(String createdBefore) {
        this.createdBefore = createdBefore;
    }

    public String getUpdatedAfter() {
        return updatedAfter;
    }

    public void setUpdatedAfter(String updatedAfter) {
        this.updatedAfter = updatedAfter;
    }

    public String getUpdatedBefore() {
        return updatedBefore;
    }

    public void setUpdatedBefore(String updatedBefore) {
        this.updatedBefore = updatedBefore;
    }

    public String getTags() {
        return tags;
    }

    public void setTags(String tags) {
        this.tags = tags;
    }

    public String getCommentWords() {
        return commentWords;
    }

    public void setCommentWords(String commentWords) {
        this.commentWords = commentWords;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public String getResultType() {
        return resultType;
    }

    public void setResultType(String resultType) {
        this.resultType = resultType;
    }

    public List getResourceDataList() {
        return resourceDataList;
    }

    public void setResourceDataList(List resourceDataList) {
        this.resourceDataList = resourceDataList;
    }

    public void setPropertyName(String propertyName) {
        this.propertyName = propertyName;
    }

    public void setPropertyValue(String propertyValue) {
        this.propertyValue = propertyValue;
    }

    /**
     * Converts given strings to Dates
     *
     * @param dateString Allowed format mm/dd/yyyy
     *
     * @return Date corresponding to the given string date
     */
    private Date computeDate(String dateString) throws RegistryException {

        if (dateString == null || dateString.length() == 0) {
            return null;
        }

        DateFormat formatter = DateFormat.getDateInstance(DateFormat.SHORT);
        try {
            return formatter.parse(dateString);
        } catch (ParseException e) {
            String msg = "Date format is invalid: " + dateString;
            throw new RegistryException(msg, e);
        }
    }

    private void calculateAverageStars(ResourceData resourceData) {

        float tempRating = resourceData.getAverageRating() * 1000;
        tempRating = Math.round(tempRating);
        tempRating = tempRating / 1000;
        resourceData.setAverageRating(tempRating);

        float averageRating = resourceData.getAverageRating();
        String[] averageStars = new String[5];

        for (int i = 0; i < 5; i++) {

            if (averageRating >= i + 1) {
                averageStars[i] = "04";

            } else if (averageRating <= i) {
                averageStars[i] = "00";

            } else {

                float fraction = averageRating - i;

                if (fraction <= 0.125) {
                    averageStars[i] = "00";

                } else if (fraction > 0.125 && fraction <= 0.375) {
                    averageStars[i] = "01";

                } else if (fraction > 0.375 && fraction <= 0.625) {
                    averageStars[i] = "02";

                } else if (fraction > 0.625 && fraction <= 0.875) {
                    averageStars[i] = "03";

                } else {
                    averageStars[i] = "04";

                }
            }
        }

        resourceData.setAverageStars(averageStars);
    }
}
