/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.*;
import org.wso2.registry.Collection;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.servlet.utils.UserUtil;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.accesscontrol.AccessControlConstants;
import org.wso2.registry.web.actions.utils.Permission;
import org.wso2.registry.web.actions.utils.WebResourcePath;
import org.wso2.registry.web.utils.CommonUtil;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

public class ResourceDetailsAction extends AbstractRegistryAction {

    private String path;
    private Resource resource;

    /**
     * This class may contain details of the current version of resources or older versions of
     * resources. When browsing an older version of a resource, it may be necessary to get the
     * path of the current version of that resource. This field contains the path of the
     * current (active) version of the resource. If this object contains the details of the
     * current version, path feild and the activeResourcePath field will be identical.
     */
    private String activeResourcePath;

    private String relativePath;
    private String contentPath;
    private String permalink;
    private String name;
    private String author;
    private String lastUpdater;
    private String description;
    private Date createdOn;
    private Date lastModified;
    private String mediaType;
    private boolean collection;
    private boolean putAllowed;
    private boolean deleteAllowed;
    private boolean authorizeAllowed;
    private Properties properties;
    private boolean versionView;


    private List tags = new ArrayList();
    private List comments = new ArrayList();
    private List associations = new ArrayList();
    private float averageRating;
    private List userPermissions = new ArrayList();
    private List rolePermissions = new ArrayList();

    private List navigatablePaths = new ArrayList();

    private Map<String, String[]> availableActions = new HashMap<String, String[]>();

    private String parentPath;
    private String[] aspectsToAdd;

    private String UIContent;
    private String contentString;
    private String UIAction;

    /**
     * Used to populate the users in the userPermissions section
     */
    private List userNames = new ArrayList();

    /**
     * Used to populate the roles in the rolePermissions section
     */
    private List roleNames = new ArrayList();

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        Registry registry = getRegistry();

        // if path is not set, we assume that root "/" is requested
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
        }

        if (path.equals(RegistryConstants.ROOT_PATH)) {

            WebResourcePath resourcePath = new WebResourcePath();
            resourcePath.setNavigateName("/");
            resourcePath.setNavigatePath("#");

            navigatablePaths.add(resourcePath);

        } else {

            // first add the root path
            WebResourcePath rootPath = new WebResourcePath();
            rootPath.setNavigateName("/");
            rootPath.setNavigatePath(RegistryConstants.ROOT_PATH);
            navigatablePaths.add(rootPath);

            String preparedPath = path;
            if (preparedPath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.substring(RegistryConstants.PATH_SEPARATOR.length());
            }

            if (preparedPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.
                        substring(0, preparedPath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            String[] parts = preparedPath.split(RegistryConstants.PATH_SEPARATOR);
            for (int i = 0; i < parts.length; i++) {
                WebResourcePath resourcePath = new WebResourcePath();

                if (i == parts.length - 1) {
                    String[] q = parts[i].split(RegistryConstants.URL_SEPARATOR);
                    if (q.length == 2) {
                        if (q[1].startsWith("version:")) {
                            String versionNumber = q[1].substring("version:".length());
                            String navName = q[0] + " (version " + versionNumber + ")";
                            resourcePath.setNavigateName(navName);
                            this.versionView = true;
                        } else {
                            resourcePath.setNavigateName(parts[i]);
                        }
                    } else {
                        resourcePath.setNavigateName(parts[i]);
                    }
                } else {
                    resourcePath.setNavigateName(parts[i]);
                }

                String tempPath = "";
                for (int j = 0; j < i + 1; j++) {
                    tempPath = tempPath + RegistryConstants.PATH_SEPARATOR + parts[j];
                }
                resourcePath.setNavigatePath(tempPath);
                navigatablePaths.add(resourcePath);
            }
        }

        if (RegistryConstants.ROOT_PATH.equals(path)) {
            relativePath = "";
        } else {
            if (path.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                relativePath = path.substring(1, path.length());
            }
        }

        if (path.equals(RegistryConstants.ROOT_PATH)) {
            name = "root";
        } else {
            String[] parts = path.split(RegistryConstants.PATH_SEPARATOR);
            name = parts[parts.length - 1];

            String[] q = name.split("\\?");
            if (q.length == 2) {
                if (q[1].startsWith("v=")) {
                    String versionNumber = q[1].substring("v=".length());
                    name = q[0] + " (version " + versionNumber + ")";
                }
            }
        }

        contentPath = path;
        //String[] qParts = path.split("\\?");
        //if (qParts.length > 1) {
        //    contentPath = path + "&content";
        //} else {
        //    contentPath = path + "?content";
        //}

        //if (!(path.endsWith(RegistryConstants.VIEW_ACTION) || path.endsWith(RegistryConstants.EDIT_ACTION))) {
        //    path = path + RegistryConstants.VIEW_ACTION;
        //}

        if (path.endsWith(RegistryConstants.VIEW_ACTION)) {
            UIAction = RegistryConstants.BROWSE_PROPERTY;
        } else if (path.endsWith(RegistryConstants.EDIT_ACTION)) {
            UIAction = RegistryConstants.EDIT_PROPERTY;
        }

        resource = registry.get(path);

        UIContent = resource.getProperty(RegistryConstants.UI_CONTENT_PROPERTY);
        if (UIContent != null && !RegistryConstants.UI_NONE.equals(UIContent)) {
            contentString = (String) resource.getContent();
        }

        if (!versionView) {
            permalink = resource.getPermanentPath();
        } else {
            permalink = path;
        }

        if (versionView) {

            String[] parts = path.split(";version:");
            if (parts.length == 2) {
                activeResourcePath = parts[0];
            } else {
                activeResourcePath = path;
            }

        } else {
            activeResourcePath = path;
        }

        if (resource.getAuthorUserName() != null) {
            this.author = resource.getAuthorUserName();
        }

        if (resource.getDescription() != null) {
            this.description = resource.getDescription();
        }

        this.collection = resource instanceof Collection;

        this.path = resource.getPath();
        this.createdOn = resource.getCreatedTime();
        this.lastModified = resource.getLastModified();
        this.lastUpdater = resource.getLastUpdaterUserName();
        this.mediaType = resource.getMediaType();
        this.averageRating = registry.getAverageRating(resource.getPath());
        this.properties = resource.getProperties();

        Association[] asso = registry.getAllAssociations(resource.getPath());
        associations.addAll(Arrays.asList(asso));


        // todo: aspect
        List<String> aspects = resource.getAspects();
        if (aspects != null) {
            for (String aspect : aspects) {
                String[] actions = registry.getAspectActions(path, aspect);
                if (actions == null) continue;
                availableActions.put(aspect, actions);
            }
        }

        Tag[] t = registry.getTags(resource.getPath());
        tags.addAll(Arrays.asList(t));

        Comment[] c = registry.getComments(resource.getPath());
        comments.addAll(Arrays.asList(c));

        // construct the parent path
        if (path.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = "root";
        } else {
            String formattedPath = path;
            if (path.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = path.substring(
                        0, path.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        try {

            UserRealm userRealm = ((UserRegistry) registry).getUserRealm();
            String[] userArray = userRealm.getUserStoreReader().getAllUserNames();
            userNames.addAll(Arrays.asList(userArray));

            String[] rolesArray = userRealm.getUserStoreReader().getAllRoleNames();
            roleNames.addAll(Arrays.asList(rolesArray));

            String authorizationPath = path;
            if (path.indexOf("?") > 0) {
                authorizationPath = path.split("\\?")[0];
            } else if (path.indexOf(RegistryConstants.URL_SEPARATOR) > 0) {
                authorizationPath = path.split("\\;")[0];
            }

            if (UserUtil.isPutAllowed(getUserName(), authorizationPath, request)) {
                putAllowed = true;
            }

            if (UserUtil.isDeleteAllowed(getUserName(), authorizationPath, request)) {
                deleteAllowed = true;
            }

            if (UserUtil.isAuthorizeAllowed(getUserName(), authorizationPath, request)) {
                authorizeAllowed = true;
            }

            Map userPermissionMap = new HashMap();

            String[] raUsers = userRealm.getAuthorizer().getAllowedUsersForResource(path, ActionConstants.GET);
            for (String raUser : raUsers) {
                if (userPermissionMap.containsKey(raUser)) {
                    Permission permission = (Permission) userPermissionMap.get(raUser);
                    permission.setReadAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(raUser);
                    permission.setReadAllow(true);
                    userPermissionMap.put(raUser, permission);
                }
            }

            String[] rdUsers = userRealm.getAuthorizer().getDeniedUsersForResource(path, ActionConstants.GET);
            for (String rdUser : rdUsers) {
                if (userPermissionMap.containsKey(rdUser)) {
                    Permission permission = (Permission) userPermissionMap.get(rdUser);
                    permission.setReadDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(rdUser);
                    permission.setReadDeny(true);
                    userPermissionMap.put(rdUser, permission);
                }
            }

            String[] waUsers = userRealm.getAuthorizer().getAllowedUsersForResource(path, ActionConstants.PUT);
            for (String waUser : waUsers) {
                if (userPermissionMap.containsKey(waUser)) {
                    Permission permission = (Permission) userPermissionMap.get(waUser);
                    permission.setWriteAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(waUser);
                    permission.setWriteAllow(true);
                    userPermissionMap.put(waUser, permission);
                }
            }

            String[] wdUsers = userRealm.getAuthorizer().getDeniedUsersForResource(path, ActionConstants.PUT);
            for (String wdUser : wdUsers) {
                if (userPermissionMap.containsKey(wdUser)) {
                    Permission permission = (Permission) userPermissionMap.get(wdUser);
                    permission.setWriteDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(wdUser);
                    permission.setWriteDeny(true);
                    userPermissionMap.put(wdUser, permission);
                }
            }

            String[] daUsers = userRealm.getAuthorizer().getAllowedUsersForResource(path, ActionConstants.DELETE);
            for (String daUser : daUsers) {
                if (userPermissionMap.containsKey(daUser)) {
                    Permission permission = (Permission) userPermissionMap.get(daUser);
                    permission.setDeleteAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(daUser);
                    permission.setDeleteAllow(true);
                    userPermissionMap.put(daUser, permission);
                }
            }

            String[] ddUsers = userRealm.getAuthorizer().getDeniedUsersForResource(path, ActionConstants.DELETE);
            for (String ddUser : ddUsers) {
                if (userPermissionMap.containsKey(ddUser)) {
                    Permission permission = (Permission) userPermissionMap.get(ddUser);
                    permission.setDeleteDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(ddUser);
                    permission.setDeleteDeny(true);
                    userPermissionMap.put(ddUser, permission);
                }
            }

            String[] aaUsers = userRealm.getAuthorizer().getAllowedUsersForResource(path, AccessControlConstants.AUTHORIZE);
            for (String aaUser : aaUsers) {
                if (userPermissionMap.containsKey(aaUser)) {
                    Permission permission = (Permission) userPermissionMap.get(aaUser);
                    permission.setAuthorizeAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(aaUser);
                    permission.setAuthorizeAllow(true);
                    userPermissionMap.put(aaUser, permission);
                }
            }

            String[] adUsers = userRealm.getAuthorizer().getDeniedUsersForResource(path, AccessControlConstants.AUTHORIZE);
            for (String adUser : adUsers) {
                if (userPermissionMap.containsKey(adUser)) {
                    Permission permission = (Permission) userPermissionMap.get(adUser);
                    permission.setAuthorizeDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(adUser);
                    permission.setAuthorizeDeny(true);
                    userPermissionMap.put(adUser, permission);
                }
            }

            userPermissions = new ArrayList(userPermissionMap.values());

            Map rolePermissionMap = new HashMap();

            String[] raRoles = userRealm.getAuthorizer().getAllowedRolesForResource(path, ActionConstants.GET);
            for (String raRole : raRoles) {
                if (rolePermissionMap.containsKey(raRole)) {
                    Permission permission = (Permission) rolePermissionMap.get(raRole);
                    permission.setReadAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(raRole);
                    permission.setReadAllow(true);
                    rolePermissionMap.put(raRole, permission);
                }
            }

            String[] rdRoles = userRealm.getAuthorizer().getDeniedRolesForResource(path, ActionConstants.GET);
            for (String rdRole : rdRoles) {
                if (rolePermissionMap.containsKey(rdRole)) {
                    Permission permission = (Permission) rolePermissionMap.get(rdRole);
                    permission.setReadDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(rdRole);
                    permission.setReadDeny(true);
                    rolePermissionMap.put(rdRole, permission);
                }
            }

            String[] waRoles = userRealm.getAuthorizer().getAllowedRolesForResource(path, ActionConstants.PUT);
            for (String waRole : waRoles) {
                if (rolePermissionMap.containsKey(waRole)) {
                    Permission permission = (Permission) rolePermissionMap.get(waRole);
                    permission.setWriteAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(waRole);
                    permission.setWriteAllow(true);
                    rolePermissionMap.put(waRole, permission);
                }
            }

            String[] wdRoles = userRealm.getAuthorizer().getDeniedRolesForResource(path, ActionConstants.PUT);
            for (String wdRole : wdRoles) {
                if (rolePermissionMap.containsKey(wdRole)) {
                    Permission permission = (Permission) rolePermissionMap.get(wdRole);
                    permission.setWriteDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(wdRole);
                    permission.setWriteDeny(true);
                    rolePermissionMap.put(wdRole, permission);
                }
            }

            String[] daRoles = userRealm.getAuthorizer().getAllowedRolesForResource(path, ActionConstants.DELETE);
            for (String daRole : daRoles) {
                if (rolePermissionMap.containsKey(daRole)) {
                    Permission permission = (Permission) rolePermissionMap.get(daRole);
                    permission.setDeleteAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(daRole);
                    permission.setDeleteAllow(true);
                    rolePermissionMap.put(daRole, permission);
                }
            }

            String[] ddRoles = userRealm.getAuthorizer().getDeniedRolesForResource(path, ActionConstants.DELETE);
            for (String ddRole : ddRoles) {
                if (rolePermissionMap.containsKey(ddRole)) {
                    Permission permission = (Permission) rolePermissionMap.get(ddRole);
                    permission.setDeleteDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(ddRole);
                    permission.setDeleteDeny(true);
                    rolePermissionMap.put(ddRole, permission);
                }
            }

            String[] aaRoles = userRealm.getAuthorizer().getAllowedRolesForResource(path, AccessControlConstants.AUTHORIZE);
            for (String aaRole : aaRoles) {
                if (rolePermissionMap.containsKey(aaRole)) {
                    Permission permission = (Permission) rolePermissionMap.get(aaRole);
                    permission.setAuthorizeAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(aaRole);
                    permission.setAuthorizeAllow(true);
                    rolePermissionMap.put(aaRole, permission);
                }
            }

            String[] adRoles = userRealm.getAuthorizer().getDeniedRolesForResource(path, AccessControlConstants.AUTHORIZE);
            for (String adRole : adRoles) {
                if (rolePermissionMap.containsKey(adRole)) {
                    Permission permission = (Permission) rolePermissionMap.get(adRole);
                    permission.setAuthorizeDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(adRole);
                    permission.setAuthorizeDeny(true);
                    rolePermissionMap.put(adRole, permission);
                }
            }

            rolePermissions = new ArrayList(rolePermissionMap.values());
            aspectsToAdd= registry.getAvailableAspects();

        } catch (UserStoreException e) {
            String msg = "Could not get user manager details. Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        return SUCCESS;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public Resource getResource() {
        return resource;
    }

    public String getActiveResourcePath() {
        return activeResourcePath;
    }

    public void setActiveResourcePath(String activeResourcePath) {
        this.activeResourcePath = activeResourcePath;
    }

    public String getRelativePath() {
        return relativePath;
    }

    public void setRelativePath(String relativePath) {
        this.relativePath = relativePath;
    }

    public String getContentPath() {
        return contentPath;
    }

    public void setContentPath(String contentPath) {
        this.contentPath = contentPath;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getAuthor() {
        return author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public String getLastUpdater() {
        return lastUpdater;
    }

    public void setLastUpdater(String lastUpdater) {
        this.lastUpdater = lastUpdater;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Date getCreatedOn() {
        return createdOn;
    }

    public String getFormattedCreatedOn() {
        return CommonUtil.formatDate(createdOn);
    }

    public void setCreatedOn(Date createdOn) {
        this.createdOn = createdOn;
    }

    public Date getLastModified() {
        return lastModified;
    }

    public String getFormattedLastModified() {
        return CommonUtil.formatDate(lastModified);
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public String getMediaType() {
        return mediaType;
    }

    public void setMediaType(String mediaType) {
        this.mediaType = mediaType;
    }

    public boolean isCollection() {
        return collection;
    }

    public boolean isPutAllowed() {
        return putAllowed;
    }

    public void setPutAllowed(boolean putAllowed) {
        this.putAllowed = putAllowed;
    }

    public boolean isDeleteAllowed() {
        return deleteAllowed;
    }

    public void setDeleteAllowed(boolean deleteAllowed) {
        this.deleteAllowed = deleteAllowed;
    }

    public boolean isAuthorizeAllowed() {
        return authorizeAllowed;
    }

    public void setAuthorizeAllowed(boolean authorizeAllowed) {
        this.authorizeAllowed = authorizeAllowed;
    }

    public Properties getProperties() {
        return properties;
    }

    public void setProperties(Properties properties) {
        this.properties = properties;
    }

    public float getAverageRating() {
        return averageRating;
    }

    public void setAverageRating(float averageRating) {
        this.averageRating = averageRating;
    }

    public List getTags() {
        return tags;
    }

    public void setTags(List tags) {
        this.tags = tags;
    }

    public List getComments() {
        return comments;
    }

    public void setComments(List comments) {
        this.comments = comments;
    }

    public List getAssociations() {
        return associations;
    }

    public void setAssociations(List associations) {
        this.associations = associations;
    }

    public String getParentPath() {
        return parentPath;
    }

    public void setParentPath(String parentPath) {
        this.parentPath = parentPath;
    }

    public List getUserNames() {
        return userNames;
    }

    public void setUserNames(List userNames) {
        this.userNames = userNames;
    }

    public List getRoleNames() {
        return roleNames;
    }

    public void setRoleNames(List roleNames) {
        this.roleNames = roleNames;
    }

    public List getUserPermissions() {
        return userPermissions;
    }

    public void setUserPermissions(List userPermissions) {
        this.userPermissions = userPermissions;
    }

    public List getRolePermissions() {
        return rolePermissions;
    }

    public void setRolePermissions(List rolePermissions) {
        this.rolePermissions = rolePermissions;
    }

    public List getNavigatablePaths() {
        return navigatablePaths;
    }

    public void setNavigatablePaths(List navigatablePaths) {
        this.navigatablePaths = navigatablePaths;
    }

    public boolean isVersionView() {
        return versionView;
    }

    public String getPermalink() {
        return permalink;
    }

    public void setPermalink(String permalink) {
        this.permalink = permalink;
    }

    public Map<String, String[]> getAvailableActions() {
        return availableActions;
    }
    public String[] getAspectsToAdd() {
        return aspectsToAdd;
    }

    public void setAspectsToAdd(String[] aspectsToAdd) {
        this.aspectsToAdd = aspectsToAdd;
    }

    public String getUIContent() {
        return UIContent;
    }

    public String getContentString() {
        return contentString;
    }

    public String getUIAction() {
        return UIAction;
    }
}
