/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.beans.PermissionsBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.web.actions.utils.Permission;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.Authorizer;
import org.wso2.registry.users.accesscontrol.AccessControlConstants;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.ActionConstants;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.servlet.utils.UserUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

public class PermissionsBeanPopulator {

    private static final Log log = LogFactory.getLog(PermissionsBeanPopulator.class);

    public static void populate(
            HttpServletRequest request, UserRegistry userRegistry, ResourcePath resourcePath) {

        String path = resourcePath.getPath();
        String userName = userRegistry.getUserName();

        PermissionsBean permissionsBean = new PermissionsBean();

        try {
            UserRealm userRealm = userRegistry.getUserRealm();

            List <String> userNames = new ArrayList<String>();
            String[] userArray = userRealm.getUserStoreReader().getAllUserNames();
            userNames.addAll(Arrays.asList(userArray));
            permissionsBean.setUserNames(userNames);

            List <String> roleNames = new ArrayList<String>();
            String[] rolesArray = userRealm.getUserStoreReader().getAllRoleNames();
            roleNames.addAll(Arrays.asList(rolesArray));
            permissionsBean.setRoleNames(roleNames);

            String authorizationPath = path;
            if (path.indexOf("?") > 0) {
                authorizationPath = path.split("\\?")[0];
            } else if (path.indexOf(RegistryConstants.URL_SEPARATOR) > 0) {
                authorizationPath = path.split("\\;")[0];
            }

            if (UserUtil.isPutAllowed(userName, authorizationPath, request)) {
                permissionsBean.setPutAllowed(true);
            } else {
                permissionsBean.setPutAllowed(false);
            }

            if (UserUtil.isDeleteAllowed(userName, authorizationPath, request)) {
                permissionsBean.setDeleteAllowed(true);
            } else {
                permissionsBean.setDeleteAllowed(false);
            }

            if (UserUtil.isAuthorizeAllowed(userName, authorizationPath, request)) {
                permissionsBean.setAuthorizeAllowed(true);
            } else {
                permissionsBean.setAuthorizeAllowed(false);
            }

            permissionsBean.setUserPermissions(getUserPermissions(userRealm, path));
            permissionsBean.setRolePermissions(getRolePermissions(userRealm, path));

        } catch (Exception e) {

            String msg = "Failed to retrieve information about permissions for resource " +
                    path + ". " + e.getMessage();
            log.error(msg, e);

            CommonUtil.addErrorMessage(request, msg);
            permissionsBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.RESOURCE_PERMISSIONS_BEAN, permissionsBean);
    }

    private static List <Permission> getUserPermissions(UserRealm userRealm, String path)
            throws UserStoreException {

        Map <String, Permission> userPermissionMap = new HashMap <String, Permission> ();

        Authorizer authorizer = userRealm.getAuthorizer();

        String[] raUsers = authorizer.getAllowedUsersForResource(path, ActionConstants.GET);
        for (String raUser : raUsers) {
            if (userPermissionMap.containsKey(raUser)) {
                Permission permission = userPermissionMap.get(raUser);
                permission.setReadAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(raUser);
                permission.setReadAllow(true);
                userPermissionMap.put(raUser, permission);
            }
        }

        String[] rdUsers = authorizer.getDeniedUsersForResource(path, ActionConstants.GET);
        for (String rdUser : rdUsers) {
            if (userPermissionMap.containsKey(rdUser)) {
                Permission permission = userPermissionMap.get(rdUser);
                permission.setReadDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(rdUser);
                permission.setReadDeny(true);
                userPermissionMap.put(rdUser, permission);
            }
        }

        String[] waUsers = authorizer.getAllowedUsersForResource(path, ActionConstants.PUT);
        for (String waUser : waUsers) {
            if (userPermissionMap.containsKey(waUser)) {
                Permission permission = userPermissionMap.get(waUser);
                permission.setWriteAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(waUser);
                permission.setWriteAllow(true);
                userPermissionMap.put(waUser, permission);
            }
        }

        String[] wdUsers = authorizer.getDeniedUsersForResource(path, ActionConstants.PUT);
        for (String wdUser : wdUsers) {
            if (userPermissionMap.containsKey(wdUser)) {
                Permission permission = userPermissionMap.get(wdUser);
                permission.setWriteDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(wdUser);
                permission.setWriteDeny(true);
                userPermissionMap.put(wdUser, permission);
            }
        }

        String[] daUsers = authorizer.getAllowedUsersForResource(path, ActionConstants.DELETE);
        for (String daUser : daUsers) {
            if (userPermissionMap.containsKey(daUser)) {
                Permission permission = userPermissionMap.get(daUser);
                permission.setDeleteAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(daUser);
                permission.setDeleteAllow(true);
                userPermissionMap.put(daUser, permission);
            }
        }

        String[] ddUsers = authorizer.getDeniedUsersForResource(path, ActionConstants.DELETE);
        for (String ddUser : ddUsers) {
            if (userPermissionMap.containsKey(ddUser)) {
                Permission permission = userPermissionMap.get(ddUser);
                permission.setDeleteDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(ddUser);
                permission.setDeleteDeny(true);
                userPermissionMap.put(ddUser, permission);
            }
        }

        String[] aaUsers = authorizer.getAllowedUsersForResource(path, AccessControlConstants.AUTHORIZE);
        for (String aaUser : aaUsers) {
            if (userPermissionMap.containsKey(aaUser)) {
                Permission permission = userPermissionMap.get(aaUser);
                permission.setAuthorizeAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(aaUser);
                permission.setAuthorizeAllow(true);
                userPermissionMap.put(aaUser, permission);
            }
        }

        String[] adUsers = authorizer.getDeniedUsersForResource(path, AccessControlConstants.AUTHORIZE);
        for (String adUser : adUsers) {
            if (userPermissionMap.containsKey(adUser)) {
                Permission permission = userPermissionMap.get(adUser);
                permission.setAuthorizeDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(adUser);
                permission.setAuthorizeDeny(true);
                userPermissionMap.put(adUser, permission);
            }
        }

        return new ArrayList <Permission> (userPermissionMap.values());
    }

    private static List <Permission> getRolePermissions(UserRealm userRealm, String path)
            throws UserStoreException {

        Map <String, Permission> rolePermissionMap = new HashMap <String, Permission> ();

        Authorizer authorizer = userRealm.getAuthorizer();

        String[] raRoles = authorizer.getAllowedRolesForResource(path, ActionConstants.GET);
        for (String raRole : raRoles) {
            if (rolePermissionMap.containsKey(raRole)) {
                Permission permission = rolePermissionMap.get(raRole);
                permission.setReadAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(raRole);
                permission.setReadAllow(true);
                rolePermissionMap.put(raRole, permission);
            }
        }

        String[] rdRoles = authorizer.getDeniedRolesForResource(path, ActionConstants.GET);
        for (String rdRole : rdRoles) {
            if (rolePermissionMap.containsKey(rdRole)) {
                Permission permission = rolePermissionMap.get(rdRole);
                permission.setReadDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(rdRole);
                permission.setReadDeny(true);
                rolePermissionMap.put(rdRole, permission);
            }
        }

        String[] waRoles = authorizer.getAllowedRolesForResource(path, ActionConstants.PUT);
        for (String waRole : waRoles) {
            if (rolePermissionMap.containsKey(waRole)) {
                Permission permission = rolePermissionMap.get(waRole);
                permission.setWriteAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(waRole);
                permission.setWriteAllow(true);
                rolePermissionMap.put(waRole, permission);
            }
        }

        String[] wdRoles = authorizer.getDeniedRolesForResource(path, ActionConstants.PUT);
        for (String wdRole : wdRoles) {
            if (rolePermissionMap.containsKey(wdRole)) {
                Permission permission = rolePermissionMap.get(wdRole);
                permission.setWriteDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(wdRole);
                permission.setWriteDeny(true);
                rolePermissionMap.put(wdRole, permission);
            }
        }

        String[] daRoles = authorizer.getAllowedRolesForResource(path, ActionConstants.DELETE);
        for (String daRole : daRoles) {
            if (rolePermissionMap.containsKey(daRole)) {
                Permission permission = rolePermissionMap.get(daRole);
                permission.setDeleteAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(daRole);
                permission.setDeleteAllow(true);
                rolePermissionMap.put(daRole, permission);
            }
        }

        String[] ddRoles = authorizer.getDeniedRolesForResource(path, ActionConstants.DELETE);
        for (String ddRole : ddRoles) {
            if (rolePermissionMap.containsKey(ddRole)) {
                Permission permission = rolePermissionMap.get(ddRole);
                permission.setDeleteDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(ddRole);
                permission.setDeleteDeny(true);
                rolePermissionMap.put(ddRole, permission);
            }
        }

        String[] aaRoles = authorizer.getAllowedRolesForResource(path, AccessControlConstants.AUTHORIZE);
        for (String aaRole : aaRoles) {
            if (rolePermissionMap.containsKey(aaRole)) {
                Permission permission = rolePermissionMap.get(aaRole);
                permission.setAuthorizeAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(aaRole);
                permission.setAuthorizeAllow(true);
                rolePermissionMap.put(aaRole, permission);
            }
        }

        String[] adRoles = authorizer.getDeniedRolesForResource(path, AccessControlConstants.AUTHORIZE);
        for (String adRole : adRoles) {
            if (rolePermissionMap.containsKey(adRole)) {
                Permission permission = rolePermissionMap.get(adRole);
                permission.setAuthorizeDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(adRole);
                permission.setAuthorizeDeny(true);
                rolePermissionMap.put(adRole, permission);
            }
        }

        return new ArrayList <Permission> (rolePermissionMap.values());
    }
}
