/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.utils;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.actions.utils.ResourceData;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletContext;
import java.util.Date;
import java.util.Calendar;
import java.text.SimpleDateFormat;
import java.text.DateFormat;
import java.text.ParseException;
import java.io.PrintWriter;
import java.io.IOException;

public class CommonUtil {

    private static final Log log = LogFactory.getLog(CommonUtil.class);

    private static final String ERROR_TITILE =
            "<strong>Error while processing the request</strong><br/>";

    /**
     * Constants used for formatting dates.
     */
    private static final long ONEMINUTE = 60 * 1000;
    private static final long ONEHOUR = 60 * ONEMINUTE;
    private static final long ONEDAY = 24 * ONEHOUR;

    public static UserRegistry getUserRegistry(HttpServletRequest request)
            throws RegistryException {

        UserRegistry userRegistry =
                (UserRegistry)request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

        if (userRegistry == null) {

            // user is not logged in. create a annoymous userRegistry for the user.

            ServletContext context = request.getSession().getServletContext();

            EmbeddedRegistry embeddedRegistry =
                    (EmbeddedRegistry)context.getAttribute(RegistryConstants.REGISTRY);

            userRegistry = embeddedRegistry.getUserRegistry();
            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, userRegistry);
        }

        return userRegistry;
    }

    public static void invalidateAllSessions(HttpServletRequest request) {
        request.getSession().invalidate();
    }

    public static void sendErrorContent(HttpServletResponse response, String msg) {

        try {
            PrintWriter out = response.getWriter();

            out.println(ERROR_TITILE);
            out.println(msg);

            out.flush();
            out.close();

        } catch (IOException e) {

            String sendError = "Failed to send error content. Caused by " + e.getMessage() +
                    "\nFollowing error was not send to the UI\n" + msg;
            log.error(sendError, e);
        }
    }

    public static void addErrorMessage(HttpServletRequest request, String msg) {

        String errorMessage = (String) request.getSession().getAttribute(UIConstants.ERROR_MESSAGE);

        if (errorMessage == null) {
            errorMessage = "<li>" + msg + "</li>";
        } else {
            errorMessage = errorMessage + "<li>" + msg + "</li>";
        }

        request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, errorMessage);
    }

    public static void forwardToPage(
            HttpServletRequest request, HttpServletResponse response, String url) {

        String errorMsg = (String) request.getSession().getAttribute(UIConstants.ERROR_MESSAGE);
        if (errorMsg != null) {
            errorMsg = "<strong>Errors have occured while processing the request.</strong><br/><ul>" + errorMsg + "</ul>";
            request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, errorMsg);
        }

        try {
            request.getRequestDispatcher(url).forward(request, response);

        } catch (Exception e) {

            String msg = "Failed to generate the page " + url + ". " + e.getMessage();
            log.error(msg, e);
        }
    }

    public static String formatDate(Date dateToParse) {

        if (dateToParse == null) {
            return null;
        }
        
        Calendar now = Calendar.getInstance();
        Calendar date = Calendar.getInstance();
        date.setTime(dateToParse);

        String value = "";
        final long timeDifference = now.getTimeInMillis() - date.getTimeInMillis();
        if (timeDifference > 0 && timeDifference < ONEDAY) {
            long hours = 0;
            if (now.getTimeInMillis() - date.getTimeInMillis() > ONEHOUR) {
                hours = ((now.getTimeInMillis() - date.getTimeInMillis()) / ONEHOUR);
                value += hours + "h ";
            }
            if (hours < 6) {
                long minutes = ((now.getTimeInMillis() - date.getTimeInMillis()) / ONEMINUTE);
                value += (minutes - hours * 60) + "m ";
            }
            value += "ago";
        } else {
            SimpleDateFormat formatter = new SimpleDateFormat("dd MMM HH:MM:ss");
            value = formatter.format(dateToParse);

            if (date.get(Calendar.YEAR) != now.get(Calendar.YEAR)) {
                formatter = new SimpleDateFormat("yyyy");
                value += " " + formatter.format(dateToParse);
            }
        }
        return value;
    }

    /**
     * Converts given strings to Dates
     *
     * @param dateString Allowed format mm/dd/yyyy
     *
     * @return Date corresponding to the given string date
     */
    public static Date computeDate(String dateString) throws RegistryException {

        if (dateString == null || dateString.length() == 0) {
            return null;
        }

        DateFormat formatter = DateFormat.getDateInstance(DateFormat.SHORT);
        try {
            return formatter.parse(dateString);
        } catch (ParseException e) {
            String msg = "Date format is invalid: " + dateString;
            throw new RegistryException(msg, e);
        }
    }

    public static void populateAverageStars(ResourceData resourceData) {

        float tempRating = resourceData.getAverageRating() * 1000;
        tempRating = Math.round(tempRating);
        tempRating = tempRating / 1000;
        resourceData.setAverageRating(tempRating);

        float averageRating = resourceData.getAverageRating();
        String[] averageStars = new String[5];

        for (int i = 0; i < 5; i++) {

            if (averageRating >= i + 1) {
                averageStars[i] = "04";

            } else if (averageRating <= i) {
                averageStars[i] = "00";

            } else {

                float fraction = averageRating - i;

                if (fraction <= 0.125) {
                    averageStars[i] = "00";

                } else if (fraction > 0.125 && fraction <= 0.375) {
                    averageStars[i] = "01";

                } else if (fraction > 0.375 && fraction <= 0.625) {
                    averageStars[i] = "02";

                } else if (fraction > 0.625 && fraction <= 0.875) {
                    averageStars[i] = "03";

                } else {
                    averageStars[i] = "04";

                }
            }
        }

        resourceData.setAverageStars(averageStars);
    }

    public static String getServerBaseURL(HttpServletRequest request) {
        String reqURL = request.getRequestURL().toString();
        return reqURL.substring(0, reqURL.indexOf("/wso2registry"));
    }
}
